@@ -33,7 +33,6 @@
 #include <iostream>
 #include <sstream>
 #include <stack>
-#include "omc_msvc.h" /* For round() */
 
 #ifndef NO_LPLIB
 
@@ -47,11 +46,32 @@
 
 #endif
 
+string UnitRes::toString()
+{
+  switch (result) {
+  case UNIT_OK: return "OK";
+  case UNKNOWN_TOKEN: return "Unknown token";
+  case UNKNOWN_IDENT: return "Unknown ident";
+  case PARSE_ERROR: return "Parse error";
+  case UNIT_OFFSET_ERROR: return "Offset error";
+  case UNIT_EXPONENT_NOT_INT: return "Exponent is not an integer";
+  case UNIT_WRONG_BASE: return "Wrong base";
+  case UNIT_NOT_FOUND: return "Unit not found";
+  case PREFIX_NOT_FOUND: return "Prefix not found";
+  case INVALID_INT: return "Invalid integer";
+  case PREFIX_NOT_ALLOWED: return "Prefix not allowed";
+  case BASE_ALREADY_DEFINED: return "Base already defined";
+  case ERROR_ADDING_UNIT: return "Error adding unit";
+  case UNITS_DEFINED_WITH_DIFFERENT_EXPR:
+  default: return "Unknown error";
+  }
+}
+
 /***************************************************************************************/
 /*   CLASS: Rational                                                                   */
 /***************************************************************************************/
 
-Rational::Rational(long numerator, long denominator) {
+Rational::Rational(mmc_sint_t numerator, mmc_sint_t denominator) {
   num = numerator;
   denom = denominator;
   fixsign();
@@ -63,16 +83,16 @@ void Rational::rationalize(double r) {
 #ifndef NO_LPLIB
   const double eps = 1e-6;
   double rapp;
-  long numerator = (long) r;
-  long denominator = 1;
+  mmc_sint_t numerator = (mmc_sint_t) r;
+  mmc_sint_t denominator = 1;
   r = round(r / eps) * eps;
   do {
     rapp = (double) numerator / (double) denominator;
     denominator *= 10;
-    numerator = (long) (r * denominator);
+    numerator = (mmc_sint_t) (r * denominator);
   } while (fabs(r - rapp) > eps);
 
-  long d = gcd(numerator, denominator);
+  mmc_sint_t d = gcd(numerator, denominator);
   num = numerator / d;
   denom = denominator / d;
   //cout << "Rationalized " << r << " to " << num << " / " << denom << endl;
@@ -89,7 +109,7 @@ bool Rational::isZero() {
   return num == 0;
 }
 
-bool Rational::is(long numerator, long denominator) {
+bool Rational::is(mmc_sint_t numerator, mmc_sint_t denominator) {
   return (num == numerator) && (denom == denominator);
 }
 
@@ -129,8 +149,31 @@ Rational Rational::div(Rational q1, Rational q2) {
   return simplify(Rational(q1.num * q2.denom, q1.denom * q2.num));
 }
 
+static mmc_sint_t powint64(mmc_sint_t base, mmc_sint_t exp) {
+  int res = 1;
+  while (exp) {
+    if (exp & 1) {
+      res *= base;
+    }
+    exp >>= 1;
+    base *= base;
+  }
+  return res;
+}
+
+Rational Rational::pow(Rational q1, Rational q2) {
+  if (q2.denom != 1) {
+    MMC_THROW();
+  }
+  if (q2.num < 0) {
+    return simplify(Rational(powint64(q1.denom, -q2.num), powint64(q1.num, -q2.num)));
+  } else {
+    return simplify(Rational(powint64(q1.num, q2.num), powint64(q1.denom, q2.num)));
+  }
+}
+
 Rational Rational::simplify(const Rational q) {
-  long gcd = Rational::gcd(q.num, q.denom);
+  mmc_sint_t gcd = Rational::gcd(q.num, q.denom);
   Rational q2(Rational(q.num / gcd, q.denom / gcd));
   q2.fixsign();
   return q2;
@@ -143,9 +186,9 @@ void Rational::fixsign() {
   }
 }
 
-long Rational::gcd(long a, long b) {
+mmc_sint_t Rational::gcd(mmc_sint_t a, mmc_sint_t b) {
   while (b != 0) {
-    long t = b;
+    mmc_sint_t t = b;
     b = a % b;
     a = t;
   }
@@ -222,11 +265,12 @@ UnitRes Unit::pow(Unit u, const Rational e, Unit& ur) {
   if (!u.offset.isZero())
     return UnitRes(UnitRes::UNIT_OFFSET_ERROR);
 
-  if (!u.scaleFactor.is(1))
-    return UnitRes(UnitRes::UNIT_SCALE_ERROR);
+  if (e.denom != 1)
+    return UnitRes(UnitRes::UNIT_EXPONENT_NOT_INT);
 
   ur = u;
   ur.prefixExpo = Rational::mul(u.prefixExpo, e);
+  ur.scaleFactor = Rational::pow(u.scaleFactor, e);
   ur.unitVec.clear();
   for (unsigned int i = 0; i < u.unitVec.size(); i++) {
     ur.unitVec.push_back(Rational::mul(u.unitVec[i], e));
@@ -288,6 +332,15 @@ void UnitParser::addPrefix(const string symbol, Rational exponent) {
   _prefix[symbol] = exponent;
 }
 
+void* UnitParser::allUnitSymbols()
+{
+  void* res = mmc_mk_nil();
+  for (map<string, Unit>::iterator p = _units.begin(); p != _units.end(); p++) {
+    res = mmc_mk_cons(mmc_mk_scon((*p).second.unitSymbol.c_str()), res);
+  }
+  return res;
+}
+
 void UnitParser::addBase(const string quantityName, const string unitName,
 
     const string unitSymbol, bool prefixAllowed) {
@@ -299,7 +352,7 @@ void UnitParser::addBase(const string quantityName, const string unitName,
     u.quantityName = b.quantityName;
     u.unitName = b.unitName;
     u.unitSymbol = unitSymbol;
-    for (unsigned long j = 0; j < _base.size(); j++) {
+    for (mmc_uint_t j = 0; j < _base.size(); j++) {
       u.unitVec.push_back(Rational((_base.size() - 1) == j ? 1 : 0));
     }
 
@@ -908,8 +961,7 @@ UnitRes UnitParser::parseFactor(Scanner& scan, Unit& unit) {
       scan.setpos(scanpostemp);
       return UnitRes(UnitRes::UNIT_OK);
     } else {
-      Unit::pow(u1, q, unit);
-      return UnitRes(UnitRes::UNIT_OK);
+      res = Unit::pow(u1, q, unit);
       return res;
     }
 
@@ -984,7 +1036,7 @@ UnitRes UnitParser::parseSymbol(Scanner& scan, Unit& unit) {
 UnitRes UnitParser::parseRational(Scanner& scan, Rational& q) {
 
   string str;
-  long l1, l2;
+  mmc_sint_t l1, l2;
   Scanner::TokenType tok = scan.getToken(str);
   if (tok == scan.TOK_INT) {
     istringstream iss1(str);
@@ -1089,7 +1141,7 @@ void UnitParser::initSIUnits() {
       Rational(1), Rational(0), true);
   addDerived("inductance", "henry", "H", "Wb/A", Rational(0), Rational(1),
       Rational(0), true);
-  addDerived("Celsius temperature", "degree Celsius", "degC", "K",
+  addDerived("thermodynamic temperature", "degree Celsius", "degC", "K",
       Rational(0), Rational(1), Rational(27315, 100), true);
   addDerived("luminous flux", "lumen", "lm", "cd.sr", Rational(0),
       Rational(1), Rational(0), true);
@@ -1106,6 +1158,59 @@ void UnitParser::initSIUnits() {
   addDerived("catalyctic activity", "katal", "kat", "s-1.mol", Rational(0),
       Rational(1), Rational(0), true);
 
+  // More derived units
+  addDerived("plane angle", "degree", "deg", "rad", Rational(0),
+      Rational(31415926535897932, 1800000000000000000), Rational(0), true);
+  addDerived("plane angle", "revolutions", "rev", "rad", Rational(0),
+      Rational(31415926535897932, 5000000000000000), Rational(0), true);
+
+  addDerived("angular velocity", "revolutions per minute", "rpm", "rad/s", Rational(0),
+      Rational(31415926535897932, 300000000000000000), Rational(0), true);
+
+  addDerived("velocity", "knot", "kn", "m/s", Rational(0),
+      Rational(1852, 3600), Rational(0), true);
+
+  addDerived("mass", "metric ton", "t", "kg", Rational(3),
+      Rational(1), Rational(0), true);
+
+  addDerived("volume", "litre", "l", "m3", Rational(0),
+      Rational(1, 1000), Rational(0), true);
+
+  addDerived("apparent power", "volt-ampere", "VA", "J/s", Rational(0),
+      Rational(1), Rational(0), true);
+  addDerived("reactive power", "volt-ampere reactive", "var", "J/s", Rational(0),
+      Rational(1), Rational(0), true);
+
+  addDerived("thermodynamic temperature", "degree Fahrenheit", "degF", "K",
+      Rational(0), Rational(5, 9), Rational(27315*9-3200*5, 900), true);
+  addDerived("thermodynamic temperature", "degree Rankine", "degRk", "K",
+      Rational(0), Rational(5, 9), Rational(0), true);
+
+  addDerived("pressure", "bar", "bar", "Pa", Rational(0), Rational(100000), Rational(0), true);
+  addDerived("pressure", "millimeter of mercury", "mmHg", "Pa", Rational(0),
+      Rational(133322387415, 1000000000), Rational(0), true);
+
+  addDerived("time", "minute", "min", "s", Rational(0), Rational(60), Rational(0), true);
+  addDerived("time", "hour", "h", "s", Rational(0), Rational(60 * 60), Rational(0), true);
+  addDerived("time", "day", "d", "s", Rational(0), Rational(60 * 60 * 24), Rational(0), true);
+
+  // Imperial units
+  addDerived("length", "inch", "in", "m", Rational(0),
+      Rational(254, 10000), Rational(0), true);
+  addDerived("length", "foot", "ft", "m", Rational(0),
+      Rational(3048, 10000), Rational(0), true);
+
+  addDerived("velocity", "miles per hour", "mph", "m/s", Rational(0),
+      Rational(44704, 100000), Rational(0), true);
+
+  addDerived("mass", "pound", "lb", "kg", Rational(0),
+      Rational(45359237, 100000000), Rational(0), true);
+
+  addDerived("pressure", "pound per square inch", "psi", "Pa", Rational(0),
+      Rational(689475729, 100000), Rational(0), true);
+  addDerived("pressure", "inch water gauge", "inWG", "Pa", Rational(0),
+      Rational(249088908333, 1000000000), Rational(0), true);
+
   commit();
 }
 
@@ -1257,4 +1362,3 @@ void TestScanner() {
   }
   cout << "\n";
 }
-
