@@ -33,6 +33,7 @@
 
 #include "../openmodelica.h"
 #include "base_array.h"
+#include "generic_array.h"
 #include "../gc/omc_gc.h"
 #include "omc_msvc.h"
 #include "index_spec.h"
@@ -86,9 +87,6 @@ static inline void clone_integer_array_spec(const integer_array_t * source,
                                             integer_array_t* dest)
 { clone_base_array_spec(source, dest); }
 
-/* Copy integer data*/
-extern void copy_integer_array_data(const integer_array_t source, integer_array_t* dest);
-
 /* Copy integer data given memory ptr*/
 extern void copy_integer_array_data_mem(const integer_array_t source,
                                         modelica_integer *dest);
@@ -161,13 +159,6 @@ extern void array_scalar_integer_array(integer_array_t* dest,int n,
 extern void array_alloc_scalar_integer_array(integer_array_t* dest,int n,
                                              modelica_integer first,...);
 
-extern modelica_integer* integer_array_element_addr(const integer_array_t * source,
-                                                    int ndims,...);
-extern modelica_integer* integer_array_element_addr1(const integer_array_t * source,
-                                                     int ndims,int dim1);
-extern modelica_integer* integer_array_element_addr2(const integer_array_t * source,int ndims,
-                                                     int dim1,int dim2);
-
 extern void cat_integer_array(int k,integer_array_t* dest, int n,
                               const integer_array_t* first,...);
 extern void cat_alloc_integer_array(int k,integer_array_t* dest, int n,
@@ -267,7 +258,7 @@ extern void cross_integer_array(const integer_array_t * x,const integer_array_t
 extern void cross_alloc_integer_array(const integer_array_t * x,const integer_array_t * y,integer_array_t* dest);
 extern void skew_integer_array(const integer_array_t * x,integer_array_t* dest);
 
-extern _index_t* integer_array_make_index_array(const integer_array_t *arr);
+extern _index_t* integer_array_make_index_array(const integer_array_t arr);
 
 static inline void clone_reverse_integer_array_spec(const integer_array_t * source,
                                                     integer_array_t* dest)
