@@ -37,6 +37,12 @@
   extern "C" {
 #endif
 
+#define EXPANDSTRING(s) EXPANDSTRINGHELPER(s)
+#define EXPANDSTRINGHELPER(s) #s
+
+#define DEFAULT_FLAG_LSS_MIN_SIZE 201
+#define DEFAULT_FLAG_NLS_MIN_SIZE 10001
+
 enum _FLAG
 {
   FLAG_UNKNOWN = 0,
@@ -46,6 +52,9 @@ enum _FLAG
   FLAG_CLOCK,
   FLAG_CPU,
   FLAG_CSV_OSTEP,
+  FLAG_CVODE_ITER,
+  FLAG_CVODE_LMM,
+  FLAG_DATA_RECONCILE_Cx,
   FLAG_DAE_MODE,
   FLAG_DELTA_X_LINEARIZE,
   FLAG_DELTA_X_SOLVER,
@@ -64,6 +73,7 @@ enum _FLAG
   FLAG_HOMOTOPY_MAX_TRIES,
   FLAG_HOMOTOPY_NEG_START_DIR,
   FLAG_HOMOTOPY_ON_FIRST_TRY,
+  FLAG_NO_HOMOTOPY_ON_FIRST_TRY,
   FLAG_HOMOTOPY_TAU_DEC_FACTOR,
   FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED,
   FLAG_HOMOTOPY_TAU_INC_FACTOR,
@@ -106,6 +116,7 @@ enum _FLAG
   FLAG_LSS_MAX_DENSITY,
   FLAG_LSS_MIN_SIZE,
   FLAG_LV,
+  FLAG_LV_TIME,
   FLAG_MAX_BISECTION_ITERATIONS,
   FLAG_MAX_EVENT_ITERATIONS,
   FLAG_MAX_ORDER,
@@ -180,6 +191,7 @@ enum SOLVER_METHOD
   S_IRKSCO,
   S_DASSL,
   S_IDA,
+  S_CVODE,
   S_ERKSSC,
   S_SYM_SOLVER,
   S_SYM_SOLVER_SSC,
@@ -284,11 +296,11 @@ enum JACOBIAN_METHOD
 {
   JAC_UNKNOWN = 0,
 
-  COLOREDNUMJAC,
-  INTERNALNUMJAC,
-  COLOREDSYMJAC,
-  NUMJAC,
-  SYMJAC,
+  COLOREDNUMJAC,      /* Colored numeric Jacobian */
+  INTERNALNUMJAC,     /* Internal numeric Jacobian */
+  COLOREDSYMJAC,      /* Colored symbolic Jacobian */
+  NUMJAC,             /* Non-colored numeric Jacobian */
+  SYMJAC,             /* Non-colored symbolic Jacobian */
 
   JAC_MAX
 };
@@ -296,22 +308,32 @@ enum JACOBIAN_METHOD
 extern const char *JACOBIAN_METHOD[JAC_MAX];
 extern const char *JACOBIAN_METHOD_DESC[JAC_MAX];
 
+/**
+ * @brief Linear system solver method
+ *
+ * Specify method to solve linear systems inside IDA.
+ */
 enum IDA_LS
 {
-  IDA_LS_UNKNOWN = 0,
+  IDA_LS_UNKNOWN = 0, /* Unknown method */
 
-  IDA_LS_DENSE,
-  IDA_LS_KLU,
-  IDA_LS_SPGMR,
-  IDA_LS_SPBCG,
-  IDA_LS_SPTFQMR,
+  IDA_LS_DENSE,     /* Default dense linear solver method */
+  IDA_LS_KLU,       /* KLU as linear solver method */
+  IDA_LS_SPGMR,     /* Scaled, Preconditioned, Generalized Minimum Residual iterative linear solver method */
+  IDA_LS_SPBCG,     /* Scaled, Preconditioned, Bi-Conjugate Gradient, Stabilized iterative linear solver method */
+  IDA_LS_SPTFQMR,   /* Scaled, Preconditioned, Transpose-Free Quasi-Minimum Residual iterative linear solver method */
 
-  IDA_LS_MAX
+  IDA_LS_MAX        /* Maximum number of methods available. Not a method itself! */
 };
 
 extern const char *IDA_LS_METHOD[IDA_LS_MAX];
 extern const char *IDA_LS_METHOD_DESC[IDA_LS_MAX];
 
+/**
+ * @brief Type of non-linear solver method
+ *
+ * Specify method to solve underlying non-linear systems.
+ */
 enum NLS_LS
 {
   NLS_LS_UNKNOWN = 0,
@@ -328,6 +350,11 @@ enum NLS_LS
 extern const char *NLS_LS_METHOD[NLS_LS_MAX];
 extern const char *NLS_LS_METHOD_DESC[NLS_LS_MAX];
 
+/**
+ * @brief Solver method for linear systems
+ *
+ * Will be used for implicit Runge-Kutta-Integrators.
+ */
 enum IMPRK_LS
 {
   IMPRK_LS_UNKNOWN = 0,
@@ -354,6 +381,18 @@ enum HOMOTOPY_BACKTRACE_STRATEGY
 extern const char *HOM_BACK_STRAT_NAME[HOM_BACK_STRAT_MAX];
 extern const char *HOM_BACK_STRAT_DESC[HOM_BACK_STRAT_MAX];
 
+enum FMU_FLAG
+{
+  FMU_FLAG_UNKNOWN = 0,
+
+  FMU_FLAG_SOLVER,
+  FMU_FLAG_NLS,
+
+  FMU_FLAG_MAX
+};
+
+/* Flag mapping to use the same descriptions and names for FMU.*/
+extern const int FMU_FLAG_MAP[FMU_FLAG_MAX];
 
 #if defined(__cplusplus)
   }
