@@ -13,10 +13,10 @@
 SimManager::SimManager(shared_ptr<IMixedSystem> system, Configuration* config)
   : _mixed_system      (system)
   , _config            (config)
-  , _tStops            ()
-  , _dimtimeevent      (0)
+  , _dimTimeEvent      (0)
   , _dimZeroFunc       (0)
   , _timeEventCounter  (NULL)
+  , _zeroVal           (NULL)
   , _events            (NULL)
   , _sampleCycles      (NULL)
   , _cycleCounter      (0)
@@ -28,7 +28,7 @@ SimManager::SimManager(shared_ptr<IMixedSystem> system, Configuration* config)
   , _tEnd              (0)
   , _lastCycleTime     (0)
   , _continueSimulation(false)
-  , _writeFinalState   (false)
+  , _checkTimeout(false)
 {
     _solver = _config->createSelectedSolver(system.get());
     _initialization = shared_ptr<Initialization>(new Initialization(dynamic_pointer_cast<ISystemInitialization>(_mixed_system), _solver));
@@ -62,6 +62,8 @@ SimManager::~SimManager()
 {
     if (_timeEventCounter)
         delete[] _timeEventCounter;
+    if (_zeroVal)
+        delete[] _zeroVal;
     if (_events)
         delete[] _events;
     if (_sampleCycles)
@@ -121,6 +123,7 @@ void SimManager::initialize()
     // Reset debug ID
     _dbgId = 0;
 
+    _solver->setStartTime(_tStart);
     try
     {
         // Build up system and update once
@@ -130,41 +133,52 @@ void SimManager::initialize()
     {
         LOGGER_WRITE("SimManager: Could not initialize system", LC_INIT, LL_ERROR);
         LOGGER_WRITE("SimManager: " + string(ex.what()), LC_INIT, LL_ERROR);
-        //ex << error_id(SIMMANAGER);
+        // Write current values as they might help to analyse the error
+        _solver->solve(ISolver::RECORDCALL);
         throw ModelicaSimulationError(SIMMANAGER, "Could not initialize system",
                                       string(ex.what()), LOGGER_IS_SET(LC_INIT, LL_ERROR));
     }
+    // Write initial values
+    _solver->solve(ISolver::RECORDCALL);
 
     if (_timeevent_system)
     {
-        _dimtimeevent = _timeevent_system->getDimTimeEvent();
+        _dimTimeEvent = _timeevent_system->getDimTimeEvent();
         if (_timeEventCounter)
             delete[] _timeEventCounter;
-        _timeEventCounter = new int[_dimtimeevent];
-        memset(_timeEventCounter, 0, _dimtimeevent * sizeof(int));
+        _timeEventCounter = new int[_dimTimeEvent];
+        memset(_timeEventCounter, 0, _dimTimeEvent * sizeof(int));
         // compute sampleCycles for RT simulation
         if (_config->getGlobalSettings()->useEndlessSim())
         {
             if (_sampleCycles)
                 delete[] _sampleCycles;
-            _sampleCycles = new int[_dimtimeevent];
+            _sampleCycles = new int[_dimTimeEvent];
             computeSampleCycles();
         }
     }
     else
-        _dimtimeevent = 0;
+        _dimTimeEvent = 0;
 
     // Set flag for endless simulation (if solver returns)
     _continueSimulation = _tEnd > _tStart;
 
-    // _solver->setTimeOut(_config->getGlobalSettings()->getAlarmTime());
+        if(_checkTimeout)
+        {
+        //being uncomment for labeling reduction
+        _solver->setTimeOut(_config->getGlobalSettings()->getAlarmTime());
+        }
     _dimZeroFunc = _event_system->getDimZeroFunc();
     _solverTask = ISolver::SOLVERCALL(ISolver::FIRST_CALL);
     if (_dimZeroFunc == _event_system->getDimZeroFunc())
     {
+        if (_zeroVal)
+            delete[] _zeroVal;
         if (_events)
             delete[] _events;
+        _zeroVal = new double[_dimZeroFunc];
         _events = new bool[_dimZeroFunc];
+        memset(_zeroVal, 0, _dimZeroFunc * sizeof(double));
         memset(_events, false, _dimZeroFunc * sizeof(bool));
     }
 
@@ -175,7 +189,7 @@ void SimManager::initialize()
     {
         _cycleCounter = 0;
         _resetCycle = _sampleCycles[0];
-        for (int i = 1; i < _dimtimeevent; i++)
+        for (int i = 1; i < _dimTimeEvent; i++)
             _resetCycle *= _sampleCycles[i];
         // All Events are updated every cycle. In order to have a change in timeEventCounter, the reset is set to two
         if(_resetCycle == 1)
@@ -195,7 +209,7 @@ void SimManager::runSingleStep()
 {
     // Increase time event counter
   double cycletime = _config->getGlobalSettings()->gethOutput();
-    if (_dimtimeevent && cycletime > 0.0)
+    if (_dimTimeEvent && cycletime > 0.0)
     {
 
       if (_lastCycleTime && cycletime != _lastCycleTime)
@@ -203,7 +217,7 @@ void SimManager::runSingleStep()
         else
             _lastCycleTime = cycletime;
 
-        for (int i = 0; i < _dimtimeevent; i++)
+        for (int i = 0; i < _dimTimeEvent; i++)
         {
             if (_cycleCounter % _sampleCycles[i] == 0)
                 _timeEventCounter[i]++;
@@ -223,7 +237,7 @@ void SimManager::runSingleStep()
     if (_cycleCounter == _resetCycle + 1)
     {
       _cycleCounter = 1;
-        for (int i = 0; i < _dimtimeevent; i++)
+        for (int i = 0; i < _dimTimeEvent; i++)
           _timeEventCounter[i] = 0;
     }
 }
@@ -304,6 +318,11 @@ void SimManager::stopSimulation()
         _solver->stop();
 }
 
+void SimManager::SetCheckTimeout(bool checkTimeout)
+{
+    _checkTimeout=checkTimeout;
+}
+
 void SimManager::writeProperties()
 {
   // declaration for Logging
@@ -449,8 +468,7 @@ void SimManager::writeProperties()
 
 void SimManager::runSingleProcess()
 {
-    double startTime, endTime, *zeroVal_0, *zeroVal_new;
-    int dimZeroF;
+    double startTime, endTime;
     double closestTimeEvent = 0.0;
 
     _H = _tEnd;
@@ -470,26 +488,23 @@ void SimManager::runSingleProcess()
      BOOST_LOG_SEV(simmgr_lg::get(), simmgr_normal) <<"Run single process." ; */
     LOGGER_WRITE("SimManager: Run single process", LC_SOLVER, LL_DEBUG);
 
-    memset(_timeEventCounter, 0, _dimtimeevent * sizeof(int));
-    dimZeroF = _event_system->getDimZeroFunc();
-    zeroVal_new = new double[dimZeroF];
+    memset(_timeEventCounter, 0, _dimTimeEvent * sizeof(int));
     _timeevent_system->setTime(_tStart);
-    if (_dimtimeevent)
+    if (_dimTimeEvent)
     {
-        _writeFinalState = true;
         _timeevent_system->computeTimeEventConditions(_tStart);
     }
-    _cont_system->evaluateAll(IContinuous::CONTINUOUS);      // vxworksupdate
-    _event_system->getZeroFunc(zeroVal_new);
+   // _cont_system->evaluateAll(IContinuous::CONTINUOUS);      // vxworksupdate
+    _event_system->getZeroFunc(_zeroVal);
 
     for (int i = 0; i < _dimZeroFunc; i++)
     {
-        _events[i] = bool(zeroVal_new[i]);
+        _events[i] = bool(_zeroVal[i]);
     }
     _mixed_system->handleSystemEvents(_events);
     //_cont_system->evaluateODE(IContinuous::CONTINUOUS);
     // Reset the time-events after the evaluation of handleSystemEvents()
-    if (_dimtimeevent)
+    if (_dimTimeEvent)
     {
       _timeevent_system->resetTimeConditions();
     }
@@ -526,16 +541,16 @@ void SimManager::runSingleProcess()
             }
             //prepare next step, starting from last endTime
             startTime = endTime;
-            if (_dimtimeevent)
+            if (_dimTimeEvent)
             {
               // Find all time events at the current time and compute next one
               closestTimeEvent = _timeevent_system->computeNextTimeEvents(startTime);
               _timeevent_system->computeTimeEventConditions(startTime);
 
-              _event_system->getZeroFunc(zeroVal_new);
+              _event_system->getZeroFunc(_zeroVal);
               for (int i = 0; i < _dimZeroFunc; i++)
               {
-                _events[i] = bool(zeroVal_new[i]);
+                _events[i] = bool(_zeroVal[i]);
               }
               //handleSystemEvents calls evaluateAll() at some point and evaluates the sampler conditions
               _mixed_system->handleSystemEvents(_events);
@@ -569,16 +584,6 @@ void SimManager::runSingleProcess()
                 break;
         }  // end if weiter nach Time Events
 
-        else  // Event am Schluss recorden.
-        {
-            //Final Solver Call
-            if (_writeFinalState)
-            {
-                _solverTask = ISolver::SOLVERCALL(ISolver::RECORDCALL);
-                _solver->solve(_solverTask);
-            }
-        }
-
         // Finish Simulation
         if ((!(_config->getGlobalSettings()->useEndlessSim())) || (_solver->getSolverStatus() & ISolver::SOLVERERROR) || (_solver->getSolverStatus() & ISolver::USER_STOP))
         {
@@ -592,17 +597,17 @@ void SimManager::runSingleProcess()
             _tStart = _tEnd;
             _tEnd += _H;
 
-            if (_dimtimeevent)
+            if (_dimTimeEvent)
             {
-                if (zeroVal_new)
+                if (_zeroVal)
                 {
                   closestTimeEvent = _timeevent_system->computeNextTimeEvents(startTime);
                   _timeevent_system->computeTimeEventConditions(_tEnd);
                   _cont_system->evaluateAll(IContinuous::CONTINUOUS);   // vxworksupdate
-                  _event_system->getZeroFunc(zeroVal_new);
+                  _event_system->getZeroFunc(_zeroVal);
                   for (int i = 0; i < _dimZeroFunc; i++)
                   {
-                    _events[i] = bool(zeroVal_new[i]);
+                    _events[i] = bool(_zeroVal[i]);
                   }
                   //_cont_system->evaluateODE(IContinuous::CONTINUOUS);
                   //reset time-events
@@ -614,10 +619,19 @@ void SimManager::runSingleProcess()
         }
 
     }  // end while continue
+
+    // treat terminal() and continuous events at final time
     _step_event_system->setTerminal(true);
-    _cont_system->evaluateAll(IContinuous::CONTINUOUS); //Is this really necessary? The solver should have already calculated the "final time point"
+    _cont_system->evaluateZeroFuncs(IContinuous::DISCRETE);
+    _event_system->getZeroFunc(_zeroVal);
+    for (int i = 0; i < _dimZeroFunc; i++)
+      _events[i] = bool(_zeroVal[i]);
+    _mixed_system->handleSystemEvents(_events);
+
+    // record final values
+    _solverTask = ISolver::SOLVERCALL(ISolver::RECORDCALL);
+    _solver->solve(_solverTask);
+
     LOGGER_STATUS("Finished", endTime, 0.0);
-    if (zeroVal_new)
-        delete[] zeroVal_new;
 }  // end singleprocess
 /** @} */ // end of coreSimcontroller
