@@ -71,6 +71,7 @@ class TraceabilityPage;
 class TabSettings;
 class StackFramesWidget;
 class TranslationFlagsWidget;
+class LibraryTreeItem;
 
 class OptionsDialog : public QDialog
 {
@@ -180,6 +181,7 @@ public slots:
   void saveSettings();
   void reset();
 private:
+  bool mDetectChange;
   GeneralSettingsPage *mpGeneralSettingsPage;
   LibrariesPage *mpLibrariesPage;
   TextEditorPage *mpTextEditorPage;
@@ -243,6 +245,7 @@ public:
   bool getShowProtectedClasses() {return mpShowProtectedClasses->isChecked();}
   void setShowHiddenClasses(bool value) {mpShowHiddenClasses->setChecked(value);}
   bool getShowHiddenClasses() {return mpShowHiddenClasses->isChecked();}
+  QCheckBox* getSynchronizeWithModelWidgetCheckBox() {return mpSynchronizeWithModelWidgetCheckBox;}
   QGroupBox* getEnableAutoSaveGroupBox() {return mpEnableAutoSaveGroupBox;}
   QSpinBox* getAutoSaveIntervalSpinBox() {return mpAutoSaveIntervalSpinBox;}
   int getWelcomePageView();
@@ -280,6 +283,7 @@ private:
   QSpinBox *mpLibraryIconTextLengthSpinBox;
   QCheckBox *mpShowProtectedClasses;
   QCheckBox *mpShowHiddenClasses;
+  QCheckBox *mpSynchronizeWithModelWidgetCheckBox;
   QGroupBox *mpEnableAutoSaveGroupBox;
   Label *mpAutoSaveIntervalLabel;
   QSpinBox *mpAutoSaveIntervalSpinBox;
@@ -347,6 +351,8 @@ public:
   Label *mpValueLabel;
   QLineEdit *mpVersionTextBox;
   QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
   bool mEditFlag;
 private slots:
   void addSystemLibrary();
@@ -366,6 +372,8 @@ public:
   Label *mpEncodingLabel;
   QComboBox *mpEncodingComboBox;
   QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
   bool mEditFlag;
 private slots:
   void browseUserLibraryPath();
@@ -606,6 +614,9 @@ public:
   QComboBox* getTargetBuildComboBox() {return mpTargetBuildComboBox;}
   QComboBox* getCompilerComboBox() {return mpCompilerComboBox;}
   QComboBox* getCXXCompilerComboBox() {return mpCXXCompilerComboBox;}
+#ifdef Q_OS_WIN
+  QCheckBox* getUseStaticLinkingCheckBox() {return mpUseStaticLinkingCheckBox;}
+#endif
   QCheckBox* getIgnoreCommandLineOptionsAnnotationCheckBox() {return mpIgnoreCommandLineOptionsAnnotationCheckBox;}
   QCheckBox* getIgnoreSimulationFlagsAnnotationCheckBox() {return mpIgnoreSimulationFlagsAnnotationCheckBox;}
   QCheckBox* getSaveClassBeforeSimulationCheckBox() {return mpSaveClassBeforeSimulationCheckBox;}
@@ -629,6 +640,9 @@ private:
   QComboBox *mpCompilerComboBox;
   Label *mpCXXCompilerLabel;
   QComboBox *mpCXXCompilerComboBox;
+#ifdef Q_OS_WIN
+  QCheckBox *mpUseStaticLinkingCheckBox;
+#endif
   QCheckBox *mpIgnoreCommandLineOptionsAnnotationCheckBox;
   QCheckBox *mpIgnoreSimulationFlagsAnnotationCheckBox;
   QCheckBox *mpSaveClassBeforeSimulationCheckBox;
@@ -798,6 +812,7 @@ public:
   void setPlottingViewMode(QString value);
   QString getPlottingViewMode();
   QCheckBox* getAutoScaleCheckBox() {return mpAutoScaleCheckBox;}
+  QCheckBox* getPrefixUnitsCheckbox() {return mpPrefixUnitsCheckbox;}
   void setCurvePattern(int pattern);
   int getCurvePattern();
   void setCurveThickness(qreal thickness);
@@ -814,6 +829,7 @@ private:
   OptionsDialog *mpOptionsDialog;
   QGroupBox *mpGeneralGroupBox;
   QCheckBox *mpAutoScaleCheckBox;
+  QCheckBox *mpPrefixUnitsCheckbox;
   QGroupBox *mpPlottingViewModeGroupBox;
   QRadioButton *mpPlottingTabbedViewRadioButton;
   QRadioButton *mpPlottingSubWindowViewRadioButton;
@@ -918,11 +934,13 @@ public:
   QString getFMIExportVersion();
   void setFMIExportType(QString type);
   QString getFMIExportType();
+  QString getFMIFlags();
   QLineEdit* getFMUNameTextBox() {return mpFMUNameTextBox;}
   QLineEdit* getMoveFMUTextBox() {return mpMoveFMUTextBox;}
   QGroupBox* getPlatformsGroupBox() {return mpPlatformsGroupBox;}
   QComboBox* getLinkingComboBox() {return mpLinkingComboBox;}
   QComboBox *getModelDescriptionFiltersComboBox() const {return mpModelDescriptionFiltersComboBox;}
+  QComboBox *getSolverForCoSimulationComboBox() const {return mpSolverForCoSimulationComboBox;}
   QCheckBox *getIncludeSourceCodeCheckBox() const {return mpIncludeSourceCodeCheckBox;}
   QCheckBox *getGenerateDebugSymbolsCheckBox() const {return mpGenerateDebugSymbolsCheckBox;}
   QCheckBox* getDeleteFMUDirectoryAndModelCheckBox() {return mpDeleteFMUDirectoryAndModelCheckBox;}
@@ -947,6 +965,7 @@ private:
   QPushButton *mpBrowseFMUDirectoryButton;
   QGroupBox *mpPlatformsGroupBox;
   QComboBox *mpLinkingComboBox;
+  QComboBox *mpSolverForCoSimulationComboBox;
   QComboBox *mpModelDescriptionFiltersComboBox;
   QCheckBox *mpIncludeSourceCodeCheckBox;
   QCheckBox *mpGenerateDebugSymbolsCheckBox;
@@ -1037,4 +1056,25 @@ private slots:
 
 };
 
+class DiscardLocalTranslationFlagsDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  DiscardLocalTranslationFlagsDialog(QWidget *pParent = 0);
+private:
+  Label *mpDescriptionLabel;
+  QListWidget *mpClassesWithLocalTranslationFlagsListWidget;
+  QPushButton *mpYesButton;
+  QPushButton *mpNoButton;
+  QDialogButtonBox *mpButtonBox;
+
+  void listLocalTranslationFlagsClasses(LibraryTreeItem *pLibraryTreeItem);
+private slots:
+  void selectUnSelectAll(bool checked);
+  void discardLocalTranslationFlags();
+  void showLocalTranslationFlags(QListWidgetItem *pListWidgetItem);
+public slots:
+  int exec();
+};
+
 #endif // OPTIONSDIALOG_H
