@@ -1270,6 +1270,8 @@ QString Utilities::convertUnitToSymbol(const QString displayUnit)
 {
   if (displayUnit.compare(QStringLiteral("Ohm")) == 0) {
     return QChar(937);
+  } else if (displayUnit.compare(QStringLiteral("degC")) == 0) {
+    return QString("%1C").arg(QChar(176));
   } else {
     return displayUnit;
   }
@@ -1286,6 +1288,8 @@ QString Utilities::convertSymbolToUnit(const QString symbol)
   // Greek Omega
   if (symbol.compare(QChar(937)) == 0) {
     return "Ohm";
+  } else if (symbol.compare(QString("%1C").arg(QChar(176))) == 0) {
+    return "degC";
   } else {
     return symbol;
   }
