@@ -41,6 +41,7 @@
 #include "qwt_scale_draw.h"
 #include "qwt_scale_widget.h"
 #include "qwt_text_label.h"
+#include "qwt_abstract_legend.h"
 
 using namespace OMPlot;
 
@@ -54,9 +55,10 @@ PlotWindow::PlotWindow(QStringList arguments, QWidget *parent, bool isInteractiv
   setPalette(p);
   // setup the main window widget
   setUpWidget();
+  // Keep default legend font since greek-mu for micro is not displayed correctly with monospaced font.
+  setLegendFont(mpPlot->legend()->font());
   // initialize plot by reading all parameters passed to it
-  if (arguments.size() > 1)
-  {
+  if (arguments.size() > 1) {
     initializePlot(arguments);
     mpPlot->getPlotZoomer()->setZoomBase(false);
   }
@@ -79,7 +81,7 @@ void PlotWindow::setUpWidget()
   // set the plot title
   setTitle(tr("Plot by OpenModelica"));
   // set the plot grid
-  setDetailedGrid(true);
+  setGrid("simple");
 }
 
 void PlotWindow::initializePlot(QStringList arguments)
@@ -90,27 +92,32 @@ void PlotWindow::initializePlot(QStringList arguments)
   setTitle(QString(arguments[2]));
   setGrid(QString(arguments[3]));
   QString plotType = arguments[4];
-  if(QString(arguments[5]) == "true")
+  if (QString(arguments[5]) == "true") {
     setLogX(true);
-  else if(QString(arguments[5]) == "false")
+  } else if (QString(arguments[5]) == "false") {
     setLogX(false);
-  else
-    throw PlotException("Invalid input" + arguments[6]);
-  if(QString(arguments[6]) == "true")
+  } else {
+    throw PlotException("Invalid input" + arguments[5]);
+  }
+  if (QString(arguments[6]) == "true") {
     setLogY(true);
-  else if(QString(arguments[6]) == "false")
+  } else if (QString(arguments[6]) == "false") {
     setLogY(false);
-  else
-    throw PlotException("Invalid input" + arguments[7]);
+  } else {
+    throw PlotException("Invalid input" + arguments[6]);
+  }
   setXLabel(QString(arguments[7]));
   setYLabel(QString(arguments[8]));
-  setUnit("");
-  setDisplayUnit("");
+  setXCustomLabel(getXLabel());
+  setYCustomLabel(getYLabel());
+  setXUnit("");
+  setXDisplayUnit("");
+  setYUnit("");
+  setYDisplayUnit("");
   setXRange(QString(arguments[9]).toDouble(), QString(arguments[10]).toDouble());
   setYRange(QString(arguments[11]).toDouble(), QString(arguments[12]).toDouble());
   setCurveWidth(QString(arguments[13]).toDouble());
   setCurveStyle(QString(arguments[14]).toInt());
-  setLegendFont(QApplication::font());
   setLegendPosition(QString(arguments[15]));
   setFooter(QString(arguments[16]));
   if (QString(arguments[17]) == "true") {
@@ -121,6 +128,9 @@ void PlotWindow::initializePlot(QStringList arguments)
     throw PlotException("Invalid input" + arguments[17]);
   }
   setTimeUnit("");
+  setPrefixUnits(false);
+  setCanUseXPrefixUnits(false);
+  setCanUseYPrefixUnits(false);
   /* read variables */
   QStringList variablesToRead;
   for(int i = 18; i < arguments.length(); i++)
@@ -301,54 +311,30 @@ void PlotWindow::setupToolbar()
   }
   // Auto scale
   mpAutoScaleButton = new QToolButton(toolBar);
-  mpAutoScaleButton->setText(tr("Auto Scale"));
+  QString autoScale(tr("Auto Scale"));
+  mpAutoScaleButton->setText(autoScale);
+  mpAutoScaleButton->setToolTip(autoScale);
+  mpAutoScaleButton->setStatusTip(autoScale);
+  mpAutoScaleButton->setIcon(QIcon(":/Resources/icons/auto_scale.svg"));
   mpAutoScaleButton->setCheckable(true);
   connect(mpAutoScaleButton, SIGNAL(toggled(bool)), SLOT(setAutoScale(bool)));
   toolBar->addWidget(mpAutoScaleButton);
   toolBar->addSeparator();
   //Fit in View
   QToolButton *fitInViewButton = new QToolButton(toolBar);
-  fitInViewButton->setText(tr("Fit in View"));
+  QString fitInView(tr("Fit in View"));
+  fitInViewButton->setText(fitInView);
+  fitInViewButton->setToolTip(fitInView);
+  fitInViewButton->setStatusTip(fitInView);
+  fitInViewButton->setIcon(QIcon(":/Resources/icons/fit-to-diagram.svg"));
   connect(fitInViewButton, SIGNAL(clicked()), SLOT(fitInView()));
   toolBar->addWidget(fitInViewButton);
   toolBar->addSeparator();
-  //EXPORT
-  QToolButton *btnExport = new QToolButton(toolBar);
-  btnExport->setText(tr("Save"));
-  //btnExport->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
-  connect(btnExport, SIGNAL(clicked()), SLOT(exportDocument()));
-  toolBar->addWidget(btnExport);
-  toolBar->addSeparator();
-  //PRINT
-  QToolButton *btnPrint = new QToolButton(toolBar);
-  btnPrint->setText(tr("Print"));
-  connect(btnPrint, SIGNAL(clicked()), SLOT(printPlot()));
-  toolBar->addWidget(btnPrint);
-  toolBar->addSeparator();
   //GRID
-  mpGridButton = new QToolButton(toolBar);
-  mpGridButton->setText(tr("Grid"));
-  mpGridButton->setCheckable(true);
-  connect(mpGridButton, SIGNAL(toggled(bool)), SLOT(setGrid(bool)));
-  toolBar->addWidget(mpGridButton);
-  // Detailed grid
-  mpDetailedGridButton = new QToolButton(toolBar);
-  mpDetailedGridButton->setText(tr("Detailed Grid"));
-  mpDetailedGridButton->setCheckable(true);
-  connect(mpDetailedGridButton, SIGNAL(toggled(bool)), SLOT(setDetailedGrid(bool)));
-  toolBar->addWidget(mpDetailedGridButton);
-  // No Grid Button
-  mpNoGridButton = new QToolButton(toolBar);
-  mpNoGridButton->setText(tr("No Grid"));
-  mpNoGridButton->setCheckable(true);
-  connect(mpNoGridButton, SIGNAL(toggled(bool)), SLOT(setNoGrid(bool)));
-  toolBar->addWidget(mpNoGridButton);
-  // Add grid buttons to buttons group
-  QButtonGroup *pGridButtonGroup = new QButtonGroup;
-  pGridButtonGroup->setExclusive(true);
-  pGridButtonGroup->addButton(mpGridButton);
-  pGridButtonGroup->addButton(mpDetailedGridButton);
-  pGridButtonGroup->addButton(mpNoGridButton);
+  mpGridComboBox = new QComboBox;
+  mpGridComboBox->addItems(QStringList() << tr("Grid") << tr("Detailed Grid") << tr("No Grid"));
+  connect(mpGridComboBox, SIGNAL(currentIndexChanged(int)), SLOT(setGrid(int)));
+  toolBar->addWidget(mpGridComboBox);
   toolBar->addSeparator();
   //LOG x LOG y
   mpLogXCheckBox = new QCheckBox(tr("Log X"), this);
@@ -361,9 +347,33 @@ void PlotWindow::setupToolbar()
   toolBar->addSeparator();
   // setup
   mpSetupButton = new QToolButton(toolBar);
-  mpSetupButton->setText(tr("Setup"));
+  QString setup(tr("Setup"));
+  mpSetupButton->setText(setup);
+  mpSetupButton->setToolTip(setup);
+  mpSetupButton->setStatusTip(setup);
+  mpSetupButton->setIcon(QIcon(":/Resources/icons/options.svg"));
   connect(mpSetupButton, SIGNAL(clicked()), SLOT(showSetupDialog()));
   toolBar->addWidget(mpSetupButton);
+  toolBar->addSeparator();
+  //EXPORT
+  QToolButton *btnExport = new QToolButton(toolBar);
+  QString save(tr("Save"));
+  btnExport->setText(save);
+  btnExport->setToolTip(save);
+  btnExport->setStatusTip(save);
+  btnExport->setIcon(QIcon(":/Resources/icons/save.svg"));
+  connect(btnExport, SIGNAL(clicked()), SLOT(exportDocument()));
+  toolBar->addWidget(btnExport);
+  toolBar->addSeparator();
+  //PRINT
+  QToolButton *btnPrint = new QToolButton(toolBar);
+  QString print(tr("Print"));
+  btnPrint->setText(print);
+  btnPrint->setToolTip(print);
+  btnPrint->setStatusTip(print);
+  btnPrint->setIcon(QIcon(":/Resources/icons/print.svg"));
+  connect(btnPrint, SIGNAL(clicked()), SLOT(printPlot()));
+  toolBar->addWidget(btnPrint);
   // finally add the tool bar to the mainwindow
   addToolBar(toolBar);
 }
@@ -406,7 +416,8 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
         {
           variablesPlotted.append(currentVariable);
           if (!editCase) {
-            pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), currentVariable, "time", currentVariable, getUnit(), getDisplayUnit(), mpPlot);
+            QFileInfo fileInfo(mFile);
+            pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "time", getXUnit(), getXDisplayUnit(), currentVariable, getYUnit(), getYDisplayUnit(), mpPlot);
             mpPlot->addPlotCurve(pPlotCurve);
           }
           // clear previous curve data
@@ -474,7 +485,8 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
         }
 
         if (!editCase) {
-          pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), csvReader->variables[i], "time", csvReader->variables[i], getUnit(), getDisplayUnit(), mpPlot);
+          QFileInfo fileInfo(mFile);
+          pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "time", getXUnit(), getXDisplayUnit(), csvReader->variables[i], getYUnit(), getYDisplayUnit(), mpPlot);
           mpPlot->addPlotCurve(pPlotCurve);
         }
         // clear previous curve data
@@ -526,7 +538,8 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
         variablesPlotted.append(reader.allInfo[i].name);
         // create the plot curve for variable
         if (!editCase) {
-          pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), reader.allInfo[i].name, "time", reader.allInfo[i].name, getUnit(), getDisplayUnit(), mpPlot);
+          QFileInfo fileInfo(mFile);
+          pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "time", getXUnit(), getXDisplayUnit(), reader.allInfo[i].name, getYUnit(), getYDisplayUnit(), mpPlot);
           mpPlot->addPlotCurve(pPlotCurve);
         }
         // read the variable values
@@ -564,8 +577,7 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
           pPlotCurve->addYAxisValue(val);
           pPlotCurve->addXAxisValue(stopTime);
           pPlotCurve->addYAxisValue(val);
-          pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(),
-                              pPlotCurve->getSize());
+          pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
           pPlotCurve->attach(mpPlot);
           mpPlot->replot();
         }
@@ -595,21 +607,16 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
   {
     xVariable = mVariablesList.at(pair);
     yVariable = mVariablesList.at(pair+1);
-    //    if (!editCase)
-    //    {
-    if (pair==0)
-    {
+
+    if (pair==0) {
       xTitle = xVariable;
       yTitle = yVariable;
-    }
-    else
-    {
+    } else {
       xTitle += ", "+xVariable;
       yTitle += ", "+yVariable;
     }
     setXLabel(xTitle);
     setYLabel(yTitle);
-    //    }
 
     //PLOT PLT
     if (mFile.fileName().endsWith("plt"))
@@ -645,9 +652,8 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
             if (variablesPlotted.size() == 1)
             {
               if (!editCase) {
-                pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-                pPlotCurve->setXVariable(xVariable);
-                pPlotCurve->setYVariable(yVariable);
+                QFileInfo fileInfo(mFile);
+                pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
                 mpPlot->addPlotCurve(pPlotCurve);
               }
               // clear previous curve data
@@ -719,9 +725,8 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
       }
 
       if (!editCase) {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-        pPlotCurve->setXVariable(xVariable);
-        pPlotCurve->setYVariable(yVariable);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       // clear previous curve data
@@ -753,9 +758,8 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
         throw PlotException(msg);
 
       if (!editCase) {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-        pPlotCurve->setXVariable(xVariable);
-        pPlotCurve->setYVariable(yVariable);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       //Fill variable x with data
@@ -786,7 +790,7 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
           omc_free_matlab4_reader(&reader);
           throw NoVariableException(QString("Parameter doesn't have a value : ").append(xVariable).toStdString().c_str());
         }
-        pPlotCurve->addYAxisValue(xval);
+        pPlotCurve->addXAxisValue(xval);
       }
       //Fill variable y with data
       var = omc_matlab4_find_var(&reader, yVariable.toStdString().c_str());
@@ -962,7 +966,8 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
         pPlotCurve->clearXAxisVector();
         pPlotCurve->clearYAxisVector();
       } else {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), currentVariable, "array index", currentVariable, getUnit(), getDisplayUnit(), mpPlot);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "array index", getXUnit(), getXDisplayUnit(), currentVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       QList<double> arrLst;
@@ -1003,7 +1008,8 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
     QStringList::Iterator itVarList;
     for (itVarList = mVariablesList.begin(); itVarList != mVariablesList.end(); itVarList++){
       if (!editCase) {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), *itVarList, "array index", *itVarList, getUnit(), getDisplayUnit(), mpPlot);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "array index", getXUnit(), getXDisplayUnit(), *itVarList, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       QList<double> res;
@@ -1066,7 +1072,8 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
       QStringList::Iterator itVarList;
       for (itVarList = mVariablesList.begin(); itVarList != mVariablesList.end(); itVarList++){
         if (!editCase) {
-          pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), *itVarList, "array index", *itVarList, getUnit(), getDisplayUnit(), mpPlot);
+          QFileInfo fileInfo(mFile);
+          pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), "array index", getXUnit(), getXDisplayUnit(), *itVarList, getYUnit(), getYDisplayUnit(), mpPlot);
           mpPlot->addPlotCurve(pPlotCurve);
         }
         int i = 1;
@@ -1122,21 +1129,16 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
   {
     xVariable = mVariablesList.at(pair);
     yVariable = mVariablesList.at(pair+1);
-    //    if (!editCase)
-    //    {
-    if (pair==0)
-    {
+
+    if (pair==0) {
       xTitle = xVariable;
       yTitle = yVariable;
-    }
-    else
-    {
+    } else {
       xTitle += ", "+xVariable;
       yTitle += ", "+yVariable;
     }
     setXLabel(xTitle);
     setYLabel(yTitle);
-    //    }
 
     //PLOT PLT
     //we presume time is the first dataset and array elements datasets are consequent
@@ -1176,9 +1178,8 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
         pPlotCurve->clearXAxisVector();
         pPlotCurve->clearYAxisVector();
       } else {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-        pPlotCurve->setXVariable(xVariable);
-        pPlotCurve->setYVariable(yVariable);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       //Read the values
@@ -1222,9 +1223,8 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
         throw PlotException("Time out of bounds.");
       }
       if (!editCase) {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-        pPlotCurve->setXVariable(xVariable);
-        pPlotCurve->setYVariable(yVariable);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       pPlotCurve->clearXAxisVector();
@@ -1283,9 +1283,8 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
         throw PlotException(msg);
 
       if (!editCase) {
-        pPlotCurve = new PlotCurve(QFileInfo(mFile).fileName(), QFileInfo(mFile).absoluteFilePath(), yVariable + " vs " + xVariable, xVariable, yVariable, getUnit(), getDisplayUnit(), mpPlot);
-        pPlotCurve->setXVariable(xVariable);
-        pPlotCurve->setYVariable(yVariable);
+        QFileInfo fileInfo(mFile);
+        pPlotCurve = new PlotCurve(fileInfo.fileName(), fileInfo.absoluteFilePath(), xVariable, getXUnit(), getXDisplayUnit(), yVariable, getYUnit(), getYDisplayUnit(), mpPlot);
         mpPlot->addPlotCurve(pPlotCurve);
       }
       //calculate time
@@ -1354,7 +1353,7 @@ QPair<QVector<double>*, QVector<double>*> PlotWindow::plotInteractive(PlotCurve
     throw NoVariableException(QString(tr("Could not determine the variable name!")).toStdString().c_str());
   }
   QString variableName = mVariablesList.at(0);
-  pPlotCurve = new PlotCurve(mInteractiveModelName, "", variableName, "time", variableName, getUnit(), getDisplayUnit(), mpPlot);
+  pPlotCurve = new PlotCurve(mInteractiveModelName, "", "time", getXUnit(), getXDisplayUnit(), variableName, getYUnit(), getYDisplayUnit(), mpPlot);
   // clear previous curve data
   pPlotCurve->clearXAxisVector();
   pPlotCurve->clearYAxisVector();
@@ -1406,20 +1405,28 @@ void PlotWindow::updateYAxis(QPair<double, double> minMaxValues)
   }
 }
 
+/*!
+ * \brief PlotWindow::updatePlot
+ * This function is called by OMEdit when auto scale is false.
+ * Updates the plot manually.
+ */
+void PlotWindow::updatePlot()
+{
+  mpPlot->updateLayout();
+  mpPlot->replot();
+  if (mpPlot->getPlotZoomer()->zoomStack().size() == 1) {
+    mpPlot->getPlotZoomer()->setZoomBase(false);
+  }
+}
 
 void PlotWindow::setGrid(QString grid)
 {
-  if (grid.toLower().compare("simple") == 0)
-  {
-    setGrid(true);
-  }
-  else if (grid.toLower().compare("none") == 0)
-  {
-    setNoGrid(true);
-  }
-  else
-  {
-    setDetailedGrid(true);
+  if (grid.toLower().compare("detailed") == 0) {
+    mpGridComboBox->setCurrentIndex(1);
+  } else if (grid.toLower().compare("none") == 0) {
+    mpGridComboBox->setCurrentIndex(2);
+  } else {
+    mpGridComboBox->setCurrentIndex(0);
   }
 }
 
@@ -1438,16 +1445,6 @@ QCheckBox* PlotWindow::getLogYCheckBox()
   return mpLogYCheckBox;
 }
 
-void PlotWindow::setXLabel(QString label)
-{
-  mpPlot->setAxisTitle(QwtPlot::xBottom, label);
-}
-
-void PlotWindow::setYLabel(QString label)
-{
-  mpPlot->setAxisTitle(QwtPlot::yLeft, label);
-}
-
 void PlotWindow::setXRange(double min, double max)
 {
   if (!(max == 0 && min == 0)) {
@@ -1595,6 +1592,36 @@ QString PlotWindow::getFooter()
 #endif
 }
 
+bool PlotWindow::getPrefixUnits() const
+{
+  return mPrefixUnits;
+}
+
+void PlotWindow::setPrefixUnits(bool prefixUnits)
+{
+  mPrefixUnits = prefixUnits;
+}
+
+bool PlotWindow::canUseXPrefixUnits() const
+{
+  return mCanUseXPrefixUnits;
+}
+
+void PlotWindow::setCanUseXPrefixUnits(bool canUseXPrefixUnits)
+{
+  mCanUseXPrefixUnits = canUseXPrefixUnits;
+}
+
+bool PlotWindow::canUseYPrefixUnits() const
+{
+  return mCanUseYPrefixUnits;
+}
+
+void PlotWindow::setCanUseYPrefixUnits(bool canUseYPrefixUnits)
+{
+  mCanUseYPrefixUnits = canUseYPrefixUnits;
+}
+
 void PlotWindow::checkForErrors(QStringList variables, QStringList variablesPlotted)
 {
   QStringList nonExistingVariables;
@@ -1730,37 +1757,30 @@ void PlotWindow::printPlot()
   }
 }
 
-void PlotWindow::setGrid(bool on)
+/*!
+ * \brief PlotWindow::setGrid
+ * Slot activated when grid combobox index is changed.
+ * Sets the grid type.
+ * \param index
+ */
+void PlotWindow::setGrid(int index)
 {
-  if (on)
-  {
-    mGridType = "simple";
-    mpPlot->getPlotGrid()->setGrid();
-    mpPlot->getPlotGrid()->attach(mpPlot);
-    mpGridButton->setChecked(true);
-  }
-  mpPlot->replot();
-}
-
-void PlotWindow::setDetailedGrid(bool on)
-{
-  if (on)
-  {
-    mGridType = "detailed";
-    mpPlot->getPlotGrid()->setDetailedGrid();
-    mpPlot->getPlotGrid()->attach(mpPlot);
-    mpDetailedGridButton->setChecked(true);
-  }
-  mpPlot->replot();
-}
-
-void PlotWindow::setNoGrid(bool on)
-{
-  if (on)
-  {
-    mGridType = "none";
-    mpPlot->getPlotGrid()->detach();
-    mpNoGridButton->setChecked(true);
+  switch (index) {
+    case 1:
+      mGridType = "detailed";
+      mpPlot->getPlotGrid()->setDetailedGrid();
+      mpPlot->getPlotGrid()->attach(mpPlot);
+      break;
+    case 2:
+      mGridType = "none";
+      mpPlot->getPlotGrid()->detach();
+      break;
+    case 0:
+    default:
+      mGridType = "simple";
+      mpPlot->getPlotGrid()->setGrid();
+      mpPlot->getPlotGrid()->attach(mpPlot);
+      break;
   }
   mpPlot->replot();
 }
@@ -1823,6 +1843,21 @@ void PlotWindow::setAutoScale(bool on)
   mpAutoScaleButton->blockSignals(state);
 }
 
+bool PlotWindow::toggleSign(PlotCurve *pPlotCurve, bool checked)
+{
+  bool toggleSign = false;
+  bool previousToggle = pPlotCurve->getToggleSign();
+  pPlotCurve->setToggleSign(checked);
+  if ((!previousToggle && pPlotCurve->getToggleSign()) || (previousToggle && !pPlotCurve->getToggleSign())) {
+    for (int i = 0 ; i < pPlotCurve->mYAxisVector.size() ; i++) {
+      pPlotCurve->updateYAxisValue(i, -pPlotCurve->mYAxisVector.at(i));
+    }
+    pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
+    toggleSign = true;
+  }
+  return toggleSign;
+}
+
 void PlotWindow::showSetupDialog()
 {
   SetupDialog *pSetupDialog = new SetupDialog(this);
@@ -1847,7 +1882,7 @@ VariablePageWidget::VariablePageWidget(PlotCurve *pPlotCurve, SetupDialog *pSetu
   // general group box
   mpGeneralGroupBox = new QGroupBox(tr("General"));
   mpLegendLabel = new QLabel(tr("Legend"));
-  mpLegendTextBox = new QLineEdit(mpPlotCurve->title().text());
+  mpLegendTextBox = new QLineEdit(mpPlotCurve->getCustomTitle().isEmpty() ? mpPlotCurve->title().text() : mpPlotCurve->getCustomTitle());
   mpResetLabelButton = new QPushButton(tr("Reset"));
   mpResetLabelButton->setAutoDefault(false);
   connect(mpResetLabelButton, SIGNAL(clicked()), SLOT(resetLabel()));
@@ -1893,6 +1928,9 @@ VariablePageWidget::VariablePageWidget(PlotCurve *pPlotCurve, SetupDialog *pSetu
   // hide
   mpHideCheckBox = new QCheckBox(tr("Hide"));
   mpHideCheckBox->setChecked(!mpPlotCurve->isVisible());
+  // toggle sign
+  mpToggleSignCheckBox = new QCheckBox(tr("Toggle Sign"));
+  mpToggleSignCheckBox->setChecked(mpPlotCurve->getToggleSign());
   // appearance layout
   QGridLayout *pAppearanceGroupBoxGridLayout = new QGridLayout;
   pAppearanceGroupBoxGridLayout->addWidget(mpColorLabel, 0, 0);
@@ -1903,6 +1941,7 @@ VariablePageWidget::VariablePageWidget(PlotCurve *pPlotCurve, SetupDialog *pSetu
   pAppearanceGroupBoxGridLayout->addWidget(mpThicknessLabel, 2, 0);
   pAppearanceGroupBoxGridLayout->addWidget(mpThicknessSpinBox, 2, 1, 1, 2);
   pAppearanceGroupBoxGridLayout->addWidget(mpHideCheckBox, 3, 0, 1, 3);
+  pAppearanceGroupBoxGridLayout->addWidget(mpToggleSignCheckBox, 4, 0, 1, 3);
   mpAppearanceGroupBox->setLayout(pAppearanceGroupBoxGridLayout);
   // set layout
   QGridLayout *pMainLayout = new QGridLayout;
@@ -1921,12 +1960,9 @@ void VariablePageWidget::setCurvePickColorButtonIcon()
 
 void VariablePageWidget::resetLabel()
 {
-  if (mpPlotCurve->getDisplayUnit().isEmpty()) {
-    mpLegendTextBox->setText(mpPlotCurve->getName());
-  } else {
-    mpLegendTextBox->setText(mpPlotCurve->getName() + " [" + mpPlotCurve->getDisplayUnit() + "]");
-  }
-
+  mpPlotCurve->setCustomTitle("");
+  mpPlotCurve->setTitleLocal();
+  mpLegendTextBox->setText(mpPlotCurve->title().text());
 }
 
 void VariablePageWidget::pickColor()
@@ -1965,7 +2001,7 @@ SetupDialog::SetupDialog(PlotWindow *pPlotWindow)
   foreach (PlotCurve *pPlotCurve, plotCurves) {
     mpVariablePagesStackedWidget->addWidget(new VariablePageWidget(pPlotCurve, this));
     QListWidgetItem *pListItem = new QListWidgetItem(mpVariablesListWidget);
-    pListItem->setText(pPlotCurve->getName());
+    pListItem->setText(pPlotCurve->getYVariable());
     pListItem->setData(Qt::UserRole, pPlotCurve->getNameStructure());
   }
   connect(mpVariablesListWidget, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(variableSelected(QListWidgetItem*,QListWidgetItem*)));
@@ -1986,7 +2022,7 @@ SetupDialog::SetupDialog(PlotWindow *pPlotWindow)
   mpTitleFontSizeSpinBox->setValue(mpPlotWindow->getPlot()->titleLabel()->font().pointSizeF());
   mpTitleFontSizeSpinBox->setSingleStep(1);
   mpVerticalAxisLabel = new QLabel(tr("Vertical Axis Title"));
-  mpVerticalAxisTextBox = new QLineEdit(mpPlotWindow->getPlot()->axisTitle(QwtPlot::yLeft).text());
+  mpVerticalAxisTextBox = new QLineEdit(mpPlotWindow->getYCustomLabel());
   mpVerticalAxisTitleFontSizeLabel = new QLabel("Vertical Axis Title Font Size");
   mpVerticalAxisTitleFontSizeSpinBox = new QDoubleSpinBox;
   mpVerticalAxisTitleFontSizeSpinBox->setRange(6, std::numeric_limits<double>::max());
@@ -1998,7 +2034,7 @@ SetupDialog::SetupDialog(PlotWindow *pPlotWindow)
   mpVerticalAxisNumbersFontSizeSpinBox->setValue(mpPlotWindow->getPlot()->axisWidget(QwtPlot::yLeft)->font().pointSizeF());
   mpVerticalAxisNumbersFontSizeSpinBox->setSingleStep(1);
   mpHorizontalAxisLabel = new QLabel(tr("Horizontal Axis Title"));
-  mpHorizontalAxisTextBox = new QLineEdit(mpPlotWindow->getPlot()->axisTitle(QwtPlot::xBottom).text());
+  mpHorizontalAxisTextBox = new QLineEdit(mpPlotWindow->getXCustomLabel());
   mpHorizontalAxisTitleFontSizeLabel = new QLabel("Horizontal Axis Title Font Size");
   mpHorizontalAxisTitleFontSizeSpinBox = new QDoubleSpinBox;
   mpHorizontalAxisTitleFontSizeSpinBox->setRange(6, std::numeric_limits<double>::max());
@@ -2050,6 +2086,10 @@ SetupDialog::SetupDialog(PlotWindow *pPlotWindow)
   mpLegendPositionComboBox->addItem(tr("Right"), "right");
   mpLegendPositionComboBox->addItem(tr("Bottom"), "bottom");
   mpLegendPositionComboBox->addItem(tr("Left"), "left");
+  int index = mpLegendPositionComboBox->findData(mpPlotWindow->getLegendPosition());
+  if (index > -1) {
+    mpLegendPositionComboBox->setCurrentIndex(index);
+  }
   mpLegendFontSizeLabel = new QLabel(tr("Legend Font Size"));
   mpLegendFontSizeSpinBox = new QDoubleSpinBox;
   mpLegendFontSizeSpinBox->setRange(6, std::numeric_limits<double>::max());
@@ -2099,12 +2139,15 @@ SetupDialog::SetupDialog(PlotWindow *pPlotWindow)
   mpXMaximumTextBox->setValidator(pDoubleValidator);
   mpYMinimumTextBox->setValidator(pDoubleValidator);
   mpXMaximumTextBox->setValidator(pDoubleValidator);
+  mpPrefixUnitsCheckbox = new QCheckBox(tr("Prefix Units"));
+  mpPrefixUnitsCheckbox->setChecked(mpPlotWindow->getPrefixUnits());
   // range tab layout
   QVBoxLayout *pRangeTabVerticalLayout = new QVBoxLayout;
   pRangeTabVerticalLayout->setAlignment(Qt::AlignTop);
   pRangeTabVerticalLayout->addWidget(mpAutoScaleCheckbox);
   pRangeTabVerticalLayout->addWidget(mpXAxisGroupBox);
   pRangeTabVerticalLayout->addWidget(mpYAxisGroupBox);
+  pRangeTabVerticalLayout->addWidget(mpPrefixUnitsCheckbox);
   mpRangeTab->setLayout(pRangeTabVerticalLayout);
   // add tabs
   mpSetupTabWidget->addTab(mpVariablesTab, tr("Variables"));
@@ -2148,24 +2191,26 @@ void SetupDialog::selectVariable(QString variable)
   }
 }
 
-void SetupDialog::setupPlotCurve(VariablePageWidget *pVariablePageWidget)
+bool SetupDialog::setupPlotCurve(VariablePageWidget *pVariablePageWidget)
 {
-  if (!pVariablePageWidget)
-    return;
+  if (!pVariablePageWidget) {
+    return false;
+  }
 
   PlotCurve *pPlotCurve = pVariablePageWidget->getPlotCurve();
 
   /* set the legend title */
-  pPlotCurve->setTitle(pVariablePageWidget->getLegendTextBox()->text());
+  if (pPlotCurve->getCustomTitle().isEmpty() && pPlotCurve->title().text().compare(pVariablePageWidget->getLegendTextBox()->text()) == 0) {
+    pPlotCurve->setCustomTitle("");
+  } else {
+    pPlotCurve->setCustomTitle(pVariablePageWidget->getLegendTextBox()->text());
+  }
   /* set the curve color title */
   pPlotCurve->setCustomColor(!pVariablePageWidget->getAutomaticColorCheckBox()->isChecked());
-  if (pVariablePageWidget->getAutomaticColorCheckBox()->isChecked())
-  {
+  if (pVariablePageWidget->getAutomaticColorCheckBox()->isChecked()) {
     pVariablePageWidget->setCurveColor(pPlotCurve->pen().color());
     pVariablePageWidget->setCurvePickColorButtonIcon();
-  }
-  else
-  {
+  } else {
     QPen pen = pPlotCurve->pen();
     pen.setColor(pVariablePageWidget->getCurveColor());
     pPlotCurve->setPen(pen);
@@ -2176,14 +2221,10 @@ void SetupDialog::setupPlotCurve(VariablePageWidget *pVariablePageWidget)
   /* set the curve width */
   pPlotCurve->setCurveWidth(pVariablePageWidget->getThicknessSpinBox()->value());
   /* set the curve visibility */
-  pPlotCurve->setVisible(!pVariablePageWidget->getHideCheckBox()->isChecked());
-  QwtText text = pPlotCurve->title();
-  if (pPlotCurve->isVisible()) {
-    text.setColor(QColor(Qt::black));
-  } else {
-    text.setColor(QColor(Qt::gray));
-  }
-  pPlotCurve->setTitle(text);
+  pPlotCurve->toggleVisibility(!pVariablePageWidget->getHideCheckBox()->isChecked());
+  /* set the curve toggle sign */
+  bool toggleSign = mpPlotWindow->toggleSign(pPlotCurve, pVariablePageWidget->getToggleSignCheckBox()->isChecked());
+  return toggleSign;
 }
 
 void SetupDialog::variableSelected(QListWidgetItem *current, QListWidgetItem *previous)
@@ -2216,19 +2257,26 @@ void SetupDialog::saveSetup()
 void SetupDialog::applySetup()
 {
   // set the variables attributes
+  bool requiresFitInView = false;
   for (int i = 0 ; i < mpVariablePagesStackedWidget->count() ; i++) {
-    setupPlotCurve(qobject_cast<VariablePageWidget*>(mpVariablePagesStackedWidget->widget(i)));
+    // if any of the variable requires call to fitinview because of toggle sign.
+    if (setupPlotCurve(qobject_cast<VariablePageWidget*>(mpVariablePagesStackedWidget->widget(i)))) {
+      requiresFitInView = true;
+    }
   }
   // set the font sizes. Don't move this line. We should set the font sizes before calling setLegendPosition
   mpPlotWindow->getPlot()->setFontSizes(mpTitleFontSizeSpinBox->value(), mpVerticalAxisTitleFontSizeSpinBox->value(), mpVerticalAxisNumbersFontSizeSpinBox->value(),
-                                        mpHorizontalAxisTitleFontSizeSpinBox->value(), mpHorizontalAxisNumbersFontSizeSpinBox->value(), mpFooterFontSizeSpinBox->value(), mpLegendFontSizeSpinBox->value());
+                                        mpHorizontalAxisTitleFontSizeSpinBox->value(), mpHorizontalAxisNumbersFontSizeSpinBox->value(), mpFooterFontSizeSpinBox->value(),
+                                        mpLegendFontSizeSpinBox->value());
   // set the titles
   mpPlotWindow->getPlot()->setTitle(mpPlotTitleTextBox->text());
-  mpPlotWindow->getPlot()->setAxisTitle(QwtPlot::yLeft, mpVerticalAxisTextBox->text());
-  mpPlotWindow->getPlot()->setAxisTitle(QwtPlot::xBottom, mpHorizontalAxisTextBox->text());
+  mpPlotWindow->setYCustomLabel(mpVerticalAxisTextBox->text());
+  mpPlotWindow->setXCustomLabel(mpHorizontalAxisTextBox->text());
   mpPlotWindow->setFooter(mpPlotFooterTextBox->text());
   // set the legend
   mpPlotWindow->setLegendPosition(mpLegendPositionComboBox->itemData(mpLegendPositionComboBox->currentIndex()).toString());
+  // set the prefix units
+  mpPlotWindow->setPrefixUnits(mpPrefixUnitsCheckbox->isChecked());
   // set the auto scale
   mpPlotWindow->setAutoScale(mpAutoScaleCheckbox->isChecked());
   // set the range
@@ -2240,7 +2288,13 @@ void SetupDialog::applySetup()
     mpPlotWindow->setYRange(mpYMinimumTextBox->text().toDouble(), mpYMaximumTextBox->text().toDouble());
   }
   // replot
+  mpPlotWindow->getPlot()->getXScaleDraw()->invalidateCache();
+  mpPlotWindow->getPlot()->getYScaleDraw()->invalidateCache();
+  mpPlotWindow->getPlot()->updateLayout();
   mpPlotWindow->getPlot()->replot();
+  if (requiresFitInView) {
+    mpPlotWindow->fitInView();
+  }
 }
 
 #include "util/omc_file.c"
