@@ -59,7 +59,7 @@ using namespace std;
 
 static QString omc_version_;
 static QString omhome;
-static IAEX::InputCellDelegate* delegate_;
+static IAEX::OmcInteractiveEnvironment* delegate_;
 
 //A small trick to get access to protected function in QThread.
 class SleeperThread : public QThread
@@ -213,10 +213,10 @@ void MyTextEdit::insertFromMimeData(const QMimeData *source)
 
 
 OMS::OMS( QWidget* parent )
-  : QMainWindow( parent )
+  : QMainWindow( parent ), mpSettings(getApplicationSettings())
 {
   delegate_ = 0;
-  omc_version_ = "(version)";
+  omc_version_ = IAEX::OmcInteractiveEnvironment::OMCVersion();
   statusBar();
 
   mainFrame_ = new QFrame();
@@ -235,7 +235,11 @@ OMS::OMS( QWidget* parent )
   layout_->setMargin( 0 );
   layout_->setSpacing( 5 );
 
-  fontSize_ = 11;
+  mpSettings->sync();
+  if (mpSettings->contains("FontSize"))
+    fontSize_ = mpSettings->value("FontSize").toInt();
+  else
+    fontSize_ = 11;
   createMoshEdit();
   //createMoshError();
   createAction();
@@ -244,25 +248,22 @@ OMS::OMS( QWidget* parent )
 //  connect( this, SIGNAL( emitQuit() ),
 //    qApp, SLOT( quit() ));
 
-  // start server
-  startServer();
-
   // windows stuff
   resize( 800, 600 );
   setWindowTitle( tr("OMShell - OpenModelica Shell") );
   setWindowIcon( QIcon(":/Resources/omshell-large.svg") );
 
-  // sett start message
+  // set start message
   const char* dateStr = __DATE__; // "Mmm dd yyyy", so dateStr+7 = "yyyy"
-  copyright_info_ = QString("OMShell 1.1 Copyright Open Source Modelica Consortium (OSMC) 2002-") + (dateStr+7) + "\nDistributed under OMSC-PL and GPL, see www.openmodelica.org\n\nConnected to OpenModelica " + omc_version_;
+  copyright_info_ = QString("OMShell 1.1 Copyright Open Source Modelica Consortium (OSMC) 2002-") + (dateStr+7) + "\nDistributed under OMSC-PL and GPL, see www.openmodelica.org\n\nConnected to " + omc_version_;
   cursor_.insertText( copyright_info_, textFormat_ );
   cursor_.insertText( tr("\nTo get help on using OMShell and OpenModelica, type \"help()\" and press enter.\n"), textFormat_ );
 
-
+  delegate_ = IAEX::OmcInteractiveEnvironment::getInstance();
   // create command compleation instance
-  QString openmodelica( omhome );
+  QString openmodelica( delegate_->OpenModelicaHome() );
   if( openmodelica.isEmpty() )
-    QMessageBox::critical( 0, "OMShell Error", "Could not find environment variable OPENMODELICAHOME, command completion will not work" );
+    QMessageBox::critical( 0, "OMShell Error", "Could not find installation directory path, command completion will not work. Please make sure OpenModelica is installed properly." );
 
   try
   {
@@ -294,7 +295,6 @@ OMS::OMS( QWidget* parent )
 OMS::~OMS()
 {
   delete mainFrame_;
-  delete delegate_;
   delete commandcompletion_;
 
   delete fileMenu_;
@@ -446,7 +446,7 @@ void OMS::addCommandLine()
   cursor_.movePosition( QTextCursor::End, QTextCursor::MoveAnchor );
   moshEdit_->setTextCursor( cursor_ );
 
-  // sett original text settings
+  // set original text settings
   moshEdit_->document()->setDefaultFont(QFont("Courier New", fontSize_, QFont::Normal));
   textFormat_.setFontFamily( "Courier New" );
   textFormat_.setFontWeight( QFont::Normal );
@@ -495,7 +495,7 @@ void OMS::returnPressed()
   // remove any newline
   commandText.simplified();
 
-  // if 'quit()' exit WinMosh
+  // if 'quit()' exit OMShell
   if( commandText == "quit()" )
   {
     close();
@@ -505,7 +505,6 @@ void OMS::returnPressed()
   // send command to OMC
   if( delegate_ )
   {
-eval:
     // 2006-02-02 AF, Added try-catch
     try
     {
@@ -544,13 +543,7 @@ eval:
   }
   else
   {
-    if( startServer() )
-    {
-      cursor_.insertText("[ERROR] No OMC serer started - restarted OMC\n" );
-      goto eval;
-    }
-    else
-      cursor_.insertText("[ERROR] No OMC serer started - unable to restart OMC\n" );
+    cursor_.insertText("[ERROR] No OMC serer started - unable to restart OMC\n" );
   }
 
   // add new command line
@@ -560,45 +553,7 @@ eval:
 void OMS::exceptionInEval(exception &e)
 {
   // 2006-0-09 AF, try to reconnect to OMC first.
-  try
-  {
-    delegate_->closeConnection();
-    delegate_->reconnect();
-    returnPressed();
-  }
-  catch( exception &e )
-  {
-    // unable to reconnect, ask if user want to restart omc.
-    QString msg = QString( e.what() ) + "\n\nUnable to reconnect with OMC. Do you want to restart OMC?";
-    int result = QMessageBox::critical( 0, tr("Communication Error with OMC"),
-      msg,
-      QMessageBox::Yes | QMessageBox::Default,
-      QMessageBox::No );
-
-    if( result == QMessageBox::Yes )
-    {
-      delegate_->closeConnection();
-      if( delegate_->startDelegate() )
-      {
-        // 2006-03-14 AF, wait before trying to reconnect,
-        // give OMC time to start up
-        SleeperThread::msleep( 1000 );
-
-        //delegate_->closeConnection();
-        try
-        {
-          delegate_->reconnect();
-          returnPressed();
-        }
-        catch( exception &e )
-        {
-          QMessageBox::critical( 0, tr("Communication Error"),
-            tr("<B>Unable to communication correctlly with OMC. OMShell will therefore close.</B>") );
-          close();
-        }
-      }
-    }
-  }
+  returnPressed();
 }
 
 void OMS::insertNewline()
@@ -759,7 +714,7 @@ void OMS::loadModel()
 {
   QString filename = QFileDialog::getOpenFileName(
     this,
-    "WinMosh - Load Model",
+    "OMShell - Load Model",
     QString::null,
     "Modelica files (*.mo)" );
 
@@ -789,39 +744,20 @@ void OMS::loadModelicaLibrary()
 bool OMS::exit()
 {
   // check if omc is running, if so: ask if it is ok that omc also closes.
-  try
-  {
-    if( delegate_ )
-    {
-      delegate_->closeConnection();
-      delegate_->reconnect();
+  QMessageBox *msgBox = new QMessageBox(0);
+  msgBox->setWindowTitle(tr("OMShell - Confirm Quit"));
+  msgBox->setIcon(QMessageBox::Question);
+  msgBox->setText(tr("Are you sure you want to quit?"));
+  msgBox->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+  msgBox->setDefaultButton(QMessageBox::Yes);
 
-      QMessageBox *msgBox = new QMessageBox(0);
-      msgBox->setWindowTitle(tr("Close OMC"));
-      msgBox->setIcon(QMessageBox::Question);
-      msgBox->setText(tr("OK to quit running OpenModelica Compiler process at exit?\n(Answer No if other OMShell/OMNotebook/Graphic editor is still running)"));
-      msgBox->setStandardButtons(QMessageBox::Ok | QMessageBox::No | QMessageBox::Cancel);
-      msgBox->setDefaultButton(QMessageBox::Ok);
+  int result = msgBox->exec();
 
-      int result = msgBox->exec();
-
-      if( result == QMessageBox::Ok )
-      {
-        stopServer();
-        return true;
-      }
-      else if (result == QMessageBox::No)
-      {
-          return true;
-      }
-      else if (result == QMessageBox::Cancel)
-      {
-          return false;
-      }
-    }
+  if( result == QMessageBox::Yes ) {
+    return true;
+  } else {
+    return false;
   }
-  catch(exception e)
-  {}
 }
 
 void OMS::fontSize()
@@ -838,10 +774,14 @@ void OMS::fontSize()
     font.setPointSize(fontSize_);
     moshEdit_->document()->setDefaultFont(font);
     textFormat_.setFontPointSize( fontSize_ );
+    commandSignFormat_.setFontPointSize( fontSize_ );
 
     //cursor_ = moshEdit_->textCursor();
     cursor_.clearSelection();
     moshEdit_->setTextCursor(cursor_);
+
+    mpSettings->setValue("FontSize", fontSize_);
+    mpSettings->sync();
   }
   else
   {
@@ -865,86 +805,11 @@ void OMS::print()
   // TODO: Implement print
 }
 
-bool OMS::startServer()
-{
-  bool omcNowStarted = false;
-  QString getVersionStr = "getVersion()";
-  QString getOMHomeStr = "getInstallationDirectoryPath()";
-
-  if( delegate_ == 0 )
-  {
-    try
-    {
-      delegate_ = new IAEX::OmcInteractiveEnvironment();
-      omcNowStarted = true;
-
-      // get version no
-      delegate_->evalExpression( getVersionStr );
-      omc_version_ = delegate_->getResult();
-      omc_version_.remove( "\"" );
-      // get omhome
-      delegate_->evalExpression( getOMHomeStr );
-      omhome = delegate_->getResult();
-      omhome.remove( "\"" );
-    }
-    catch( exception &e )
-    {
-      if( !IAEX::OmcInteractiveEnvironment::startOMC() )
-      {
-        QMessageBox::critical( 0, "OMC Error", "Was unable to start OMC, therefore OMShell will not work correctly." );
-      }
-      else
-      {
-        // 2006-03-14 AF, wait before trying to reconnect,
-        // give OMC time to start up
-        SleeperThread::msleep( 3000 );
-
-        delegate_ = new IAEX::OmcInteractiveEnvironment();
-        omcNowStarted = true;
-
-        // get version no
-        QString getTempStr = "getTempDirectoryPath()";
-        delegate_->evalExpression( getTempStr );
-        QString tmpDir = delegate_->getResult()+"/OpenModelica/";
-        tmpDir.remove("\"");
-        if (!QDir().exists(tmpDir)) QDir().mkdir(tmpDir);
-        tmpDir = "cd(\"" + tmpDir + "\")";
-        cout << "Temp.Dir " << tmpDir.toStdString() << std::endl;
-        delegate_->evalExpression(tmpDir);
-        cout << "cdToTempDir: " << delegate_->getResult().toStdString() << std::endl;
-
-        QString getVersionStr = "getVersion()";
-        delegate_->evalExpression( getVersionStr );
-        omc_version_ = delegate_->getResult();
-        omc_version_.remove( "\"" );
-        cout << "OMC version " << omc_version_.toStdString() << std::endl;
-
-        // get omhome
-        delegate_->evalExpression( getOMHomeStr );
-        omhome = delegate_->getResult();
-        omhome.remove( "\"" );
-        cout << "OPENMODELICAHOME: " << omhome.toStdString() << std::endl;
-      }
-    }
-  }
-
-  return omcNowStarted;
-}
-
-void OMS::stopServer()
-{
-  if( delegate_ )
-  {
-    QString quit = "quit()";
-    delegate_->evalExpression( quit );
-  }
-}
-
 void OMS::clear()
 {
   moshEdit_->clear();
 
-  // sett original text settings
+  // set original text settings
   moshEdit_->document()->setDefaultFont(QFont("Courier New", fontSize_, QFont::Normal));
 
   addCommandLine();
@@ -960,19 +825,34 @@ void OMS::closeEvent( QCloseEvent *event )
 
 void OMS::cut()
 {
-  QKeyEvent* key = new QKeyEvent( QEvent::KeyPress, Qt::Key_X, Qt::ControlModifier, "x" );
-  ((MyTextEdit*)moshEdit_)->sendKey( key );
+  QKeyEvent key = QKeyEvent( QEvent::KeyPress, Qt::Key_X, Qt::ControlModifier, "x" );
+  ((MyTextEdit*)moshEdit_)->sendKey( &key );
 }
 
 void OMS::copy()
 {
-  QKeyEvent* key = new QKeyEvent( QEvent::KeyPress, Qt::Key_C, Qt::ControlModifier, "c" );
-  ((MyTextEdit*)moshEdit_)->sendKey( key );
+  QKeyEvent key = QKeyEvent( QEvent::KeyPress, Qt::Key_C, Qt::ControlModifier, "c" );
+  ((MyTextEdit*)moshEdit_)->sendKey( &key );
 }
 
 void OMS::paste()
 {
-  QKeyEvent* key = new QKeyEvent( QEvent::KeyPress, Qt::Key_V, Qt::ControlModifier, "v" );
-  ((MyTextEdit*)moshEdit_)->sendKey( key );
+  QKeyEvent key = QKeyEvent( QEvent::KeyPress, Qt::Key_V, Qt::ControlModifier, "v" );
+  ((MyTextEdit*)moshEdit_)->sendKey( &key );
 }
 
+QString OMS::organization = "openmodelica";  /* case-sensitive string. Don't change it. Used by ini settings file. */
+QString OMS::application = "omshell"; /* case-sensitive string. Don't change it. Used by ini settings file. */
+QString OMS::utf8 = "UTF-8";
+
+QSettings* OMS::getApplicationSettings()
+{
+  static int init = 0;
+  static QSettings *pSettings;
+  if (!init) {
+    init = 1;
+    pSettings = new QSettings(QSettings::IniFormat, QSettings::UserScope, organization, application);
+    pSettings->setIniCodec(utf8.toStdString().data());
+  }
+  return pSettings;
+}
