@@ -34,6 +34,7 @@
 #include "Flags.h"
 #include "Logging.h"
 #include "Model.h"
+#include "OMSFileSystem.h"
 #include "ssd/Tags.h"
 #include "System.h"
 #include "SystemTLM.h"
@@ -41,11 +42,10 @@
 
 #include <fmilib.h>
 #include <JM/jm_portability.h>
-#include <OMSFileSystem.h>
 #include <RegEx.h>
 
 oms::ComponentFMUCS::ComponentFMUCS(const ComRef& cref, System* parentSystem, const std::string& fmuPath)
-  : oms::Component(cref, oms_component_fmu, parentSystem, fmuPath), fmuInfo(fmuPath, oms_fmi_kind_cs)
+  : oms::Component(cref, oms_component_fmu, parentSystem, fmuPath), fmuInfo(fmuPath)
 {
 }
 
@@ -71,12 +71,12 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     return NULL;
   }
 
-  filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
+  filesystem::path temp_root(parentSystem->getModel().getTempDirectory());
   filesystem::path temp_temp = temp_root / "temp";
   filesystem::path relFMUPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + ".fmu")) : filesystem::path(fmuPath);
   filesystem::path absFMUPath = temp_root / relFMUPath;
 
-  ComponentFMUCS* component = new ComponentFMUCS(cref, parentSystem, relFMUPath.string());
+  ComponentFMUCS* component = new ComponentFMUCS(cref, parentSystem, relFMUPath.generic_string());
 
   component->callbacks.malloc = malloc;
   component->callbacks.calloc = calloc;
@@ -129,12 +129,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   }
 
   // update FMU info
-  if (oms_status_ok != component->fmuInfo.update(version, component->fmu))
-  {
-    logError("Error importing FMU attributes");
-    delete component;
-    return NULL;
-  }
+  component->fmuInfo.update(version, component->fmu);
 
   component->callbackFunctions.logger = oms::fmi2logger;
   component->callbackFunctions.allocateMemory = calloc;
@@ -168,24 +163,16 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     fmi2_import_variable_t* varState = (fmi2_import_variable_t*)fmi2_import_get_real_variable_derivative_of(varReal);
     if (varState)
     {
-      fmi2_value_reference_t state_vr = fmi2_import_get_variable_vr(varState);
-      bool found = false;
-      for (size_t i = 0; i < component->allVariables.size(); i++)
-      {
-        if (state_vr == component->allVariables[i].getValueReference())
-        {
-          component->allVariables[i].markAsState();
-          found = true;
-          break;
-        }
-      }
-      if (!found)
+      // IMPORTANT: vr is not unique!!! Do lookup with proper index or name
+      size_t originalIndex = fmi2_import_get_variable_original_order(varState);
+      if (originalIndex < 0 || originalIndex >= component->allVariables.size())
       {
         logError("Couldn't find " + std::string(fmi2_import_get_variable_name(varState)));
         fmi2_import_free_variable_list(varList);
         delete component;
         return NULL;
       }
+      component->allVariables[originalIndex].markAsState();
     }
     else
     {
@@ -205,7 +192,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
     else if (v.isOutput())
     {
       component->outputs.push_back(v);
-      component->outputsGraph.addNode(Connector(oms_causality_output, v.getType(), v.getCref()));
+      component->outputsGraph.addNode(Connector(oms_causality_output, v.getType(), v.getCref(), component->getFullCref()));
     }
     else if (v.isParameter())
       component->parameters.push_back(v);
@@ -213,7 +200,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
       component->calculatedParameters.push_back(v);
 
     if (v.isInitialUnknown())
-      component->initialUnknownsGraph.addNode(Connector(v.getCausality(), v.getType(), v.getCref()));
+      component->initialUnknownsGraph.addNode(Connector(v.getCausality(), v.getType(), v.getCref(), component->getFullCref()));
 
     component->exportVariables.push_back(v.isInput() || v.isOutput());
   }
@@ -225,15 +212,15 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   int i = 1;
   int size = 1 + component->inputs.size();
   for (const auto& v : component->inputs)
-    component->connectors.push_back(new Connector(oms_causality_input, v.getType(), v.getCref(), i++/(double)size));
+    component->connectors.push_back(new Connector(oms_causality_input, v.getType(), v.getCref(), component->getFullCref(), i++/(double)size));
   i = 1;
   size = 1 + component->outputs.size();
   for (const auto& v : component->outputs)
-    component->connectors.push_back(new Connector(oms_causality_output, v.getType(), v.getCref(), i++/(double)size));
+    component->connectors.push_back(new Connector(oms_causality_output, v.getType(), v.getCref(), component->getFullCref(), i++/(double)size));
   for (const auto& v : component->parameters)
-    component->connectors.push_back(new Connector(oms_causality_parameter, v.getType(), v.getCref()));
+    component->connectors.push_back(new Connector(oms_causality_parameter, v.getType(), v.getCref(), component->getFullCref()));
   for (const auto& v : component->calculatedParameters)
-    component->connectors.push_back(new Connector(oms_causality_calculatedParameter, v.getType(), v.getCref()));
+    component->connectors.push_back(new Connector(oms_causality_calculatedParameter, v.getType(), v.getCref(), component->getFullCref()));
   component->connectors.push_back(NULL);
   component->element.setConnectors(&component->connectors[0]);
 
@@ -251,12 +238,12 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   }
 
   // parse modelDescription.xml to get start values before instantiating fmu's
-  component->values.parseModelDescription((tempDir / "modelDescription.xml").string().c_str());
+  component->values.parseModelDescription(tempDir);
 
   return component;
 }
 
-oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion)
+oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion, const Snapshot& snapshot)
 {
   ComRef cref = ComRef(node.attribute("name").as_string());
   std::string type = node.attribute("type").as_string();
@@ -284,7 +271,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
       // import connectors
       for(pugi::xml_node_iterator itConnectors = (*it).begin(); itConnectors != (*it).end(); ++itConnectors)
       {
-        component->connectors.push_back(oms::Connector::NewConnector(*itConnectors, sspVersion));
+        component->connectors.push_back(oms::Connector::NewConnector(*itConnectors, sspVersion, component->getFullCref()));
       }
     }
     else if(name == oms::ssp::Draft20180219::ssd::element_geometry)
@@ -296,8 +283,10 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
     else if(name == oms::ssp::Version1_0::ssd::parameter_bindings)
     {
       // set parameter bindings associated with the component
-      std::string tempdir = parentSystem->getModel()->getTempDirectory();
-      component->values.importFromSSD(*it, sspVersion, tempdir);
+      Values resources;
+      std::string tempdir = parentSystem->getModel().getTempDirectory();
+      resources.importFromSnapshot(*it, sspVersion, snapshot);
+      component->values.parameterResources.push_back(resources);
     }
     else
     {
@@ -313,7 +302,7 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const pugi::xml_node& node, om
   return component;
 }
 
-oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, pugi::xml_node& ssvNode) const
+oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, Snapshot& snapshot) const
 {
 #if !defined(NO_TLM)
   if (tlmbusconnectors[0])
@@ -341,42 +330,86 @@ oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, pugi::xm
         return oms_status_error;
 
   // export ParameterBindings at component level
-  if (Flags::ExportParametersInline()) // export as inline
-  {
-    values.exportToSSD(node);
-  }
-  else
-  {
-    values.exportToSSV(ssvNode); // export to ssv file
-  }
+  values.exportParameterBindings(node, snapshot);
 
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::exportToSSV(pugi::xml_node& ssvNode)
+{
+  return values.exportToSSV(ssvNode);
+}
+
 oms_status_enu_t oms::ComponentFMUCS::exportToSSVTemplate(pugi::xml_node& ssvNode)
 {
   values.exportToSSVTemplate(ssvNode, getCref());
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::exportToSSMTemplate(pugi::xml_node& ssmNode)
+{
+  values.exportToSSMTemplate(ssmNode, getCref());
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns()
 {
   if (initialUnknownsGraph.getEdges().size() > 0)
+    return logError(std::string(getCref()) + ": " + getPath() + " is already initialized");
+
+  int N=initialUnknownsGraph.getNodes().size();
+
+  // Check if initial unknowns from modelDescription.xml are the same as in initialUnknownsGraph
+  fmi2_import_variable_list_t* initialUnknowns;
+  initialUnknowns = fmi2_import_get_initial_unknowns_list(fmu);
+  size_t N_fmilib = fmi2_import_get_variable_list_size(initialUnknowns);
+  bool badInitialUnknowns = false;
+
+  if (N != N_fmilib)
   {
-    logError(std::string(getCref()) + ": " + getPath() + " is already initialized");
-    return oms_status_error;
+    if (!Flags::IgnoreInitialUnknowns())
+      logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The FMU lists " + std::to_string(N_fmilib) + " initial unknowns but actually contains " + std::to_string(N) + " initial unknowns as per the variable definitions.");
+    badInitialUnknowns = true;
   }
 
-  if (Flags::IgnoreInitialUnknowns())
+  for (size_t i=0; i < N_fmilib; ++i)
   {
-    int N=initialUnknownsGraph.getNodes().size();
-    for (int i = 0; i < N; i++)
+    fmi2_xml_variable_t* var_fmilib = fmi2_import_get_variable(initialUnknowns, i);
+    size_t originalIndex = fmi2_import_get_variable_original_order(var_fmilib);
+    const Variable& var_oms = allVariables[originalIndex];
+    oms::ComRef name_fmilib(fmi2_import_get_variable_name(var_fmilib));
+
+    if (var_oms.getCref() != name_fmilib)
     {
-      logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all");
-      for (int j = 0; j < inputs.size(); j++)
-        initialUnknownsGraph.addEdge(inputs[j].makeConnector(), initialUnknownsGraph.getNodes()[i]);
+      if (!Flags::IgnoreInitialUnknowns())
+        logWarning("[" + std::string(getCref()) + ": " + getPath() + "] Variable " + std::string(var_oms.getCref()) + " with index " + std::to_string(originalIndex+1) + " could not be found.");
+      badInitialUnknowns = true;
+    }
+    else if (!var_oms.isInitialUnknown())
+    {
+      if (!Flags::IgnoreInitialUnknowns())
+        logWarning("[" + std::string(getCref()) + ": " + getPath() + "] Variable " + std::string(var_oms.getCref()) + " with index " + std::to_string(originalIndex+1) + " is falsely listed as initial unknown.");
+      badInitialUnknowns = true;
+    }
+  }
+
+  fmi2_import_free_variable_list(initialUnknowns);
+
+  if (badInitialUnknowns)
+  {
+    if(!Flags::IgnoreInitialUnknowns())
+      logInfo("[" + std::string(getCref()) + ": " + getPath() + "] The FMU contains bad initial unknowns. This might cause problems, e.g. wrong simulation results.");
+    else
+    {
+      logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The dependencies of the initial unknowns defined in the FMU are ignored because the flag --ignoreInitialUnknowns is active. Instead, all the initial unknowns will depend on all inputs.");
+      for (int i=0; i < N; i++)
+      {
+        logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all inputs");
+        for (int j=0; j < inputs.size(); j++)
+          initialUnknownsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
+      }
+      return oms_status_ok;
     }
-    return oms_status_ok;
   }
 
   size_t *startIndex=NULL, *dependency=NULL;
@@ -390,31 +423,46 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns(
     return oms_status_ok;
   }
 
-  int N=initialUnknownsGraph.getNodes().size();
-  for (int i = 0; i < N; i++)
+  //logInfo(std::string(getCref()) + ": " + getPath());
+  //std::string buffer = "startIndex(" + std::to_string(N+1) + ")\n";
+  //for (int i=0; i < N; i++)
+  //  buffer += "startIndex[" + std::to_string(i) + "] (" + std::string(initialUnknownsGraph.getNodes()[i]) + ") = " + std::to_string(startIndex[i]) + "\n";
+  //buffer += "startIndex[" + std::to_string(N) + "] = " + std::to_string(startIndex[N]) + "\n";
+  //logInfo(buffer);
+
+  //buffer = "dependency(" + std::to_string(startIndex[N]) + ")\n";
+  //for (int i=0; i < startIndex[N]; i++)
+  //  buffer += "dependency[" + std::to_string(i) + "] = " + std::to_string(dependency[i]) + "\n";
+  //logInfo(buffer);
+
+  //buffer = "factorKind(" + std::to_string(startIndex[N]) + ")\n";
+  //for (int i=0; i < startIndex[N]; i++)
+  //  buffer += "factorKind[" + std::to_string(i) + "] = " + std::string(fmi2_dependency_factor_kind_to_string((fmi2_dependency_factor_kind_enu_t)factorKind[i])) + "\n";
+  //logInfo(buffer);
+
+  for (int i=0; i < N_fmilib; i++)
   {
-    if (startIndex[i] == startIndex[i + 1])
+    if (startIndex[i] == startIndex[i+1])
     {
       logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " has no dependencies");
     }
-    else if ((startIndex[i] + 1 == startIndex[i + 1]) && (dependency[startIndex[i]] == 0))
+    else if ((startIndex[i] + 1 == startIndex[i+1]) && (dependency[startIndex[i]] == 0))
     {
-      logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all");
+      logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all inputs");
       for (int j = 0; j < inputs.size(); j++)
-        initialUnknownsGraph.addEdge(inputs[j].makeConnector(), initialUnknownsGraph.getNodes()[i]);
+        initialUnknownsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
     }
     else
     {
-      for (size_t j = startIndex[i]; j < startIndex[i + 1]; j++)
+      for (size_t j=startIndex[i]; j < startIndex[i+1]; j++)
       {
         if (dependency[j] < 1 || dependency[j] > allVariables.size())
         {
           logWarning("Initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " has bad dependency on variable with index " + std::to_string(dependency[j]) + " which couldn't be resolved");
-          logInfo("Use flag --ignoreInitialUnknowns=true to ignore all dependencies, but this can cause inflated loop size.");
-          return logError(std::string(getCref()) + ": erroneous dependencies detected in modelDescription.xml");
+          return logError(std::string(getCref()) + ": Erroneous initial unknowns detected in modelDescription.xml\nUse flag --ignoreInitialUnknowns=true to ignore all initial unknowns, but this can cause inflated loop size.");
         }
         logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on " + std::string(allVariables[dependency[j] - 1]));
-        initialUnknownsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(), initialUnknownsGraph.getNodes()[i]);
+        initialUnknownsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
       }
     }
   }
@@ -451,7 +499,7 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_outputs()
     {
       logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(outputs[i]) + " depends on all");
       for (int j = 0; j < inputs.size(); j++)
-        outputsGraph.addEdge(inputs[j].makeConnector(), outputs[i].makeConnector());
+        outputsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), outputs[i].makeConnector(this->getFullCref()));
     }
     else
     {
@@ -463,7 +511,7 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_outputs()
           return logError(std::string(getCref()) + ": erroneous dependencies detected in modelDescription.xml");
         }
         logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(outputs[i]) + " depends on " + std::string(allVariables[dependency[j] - 1]));
-        outputsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(), outputs[i].makeConnector());
+        outputsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(this->getFullCref()), outputs[i].makeConnector(this->getFullCref()));
       }
     }
   }
@@ -485,28 +533,35 @@ oms_status_enu_t oms::ComponentFMUCS::instantiate()
   if (jm_status_error == jmstatus)
     return logError_FMUCall("fmi2_import_instantiate", this);
 
-  // set start values
-  for (const auto& v : values.booleanStartValues)
+  // set start values from local resources
+  if (values.hasResources())
   {
-    oms::ComRef cref = getValidCref(v.first);
-    if (oms_status_ok != setBoolean(cref, v.second))
-      return logError("Failed to set start value for " + std::string(v.first));
+    for (const auto &it : values.parameterResources)
+    {
+      for (const auto &res : it.allresources)
+      {
+        setResourcesHelper1(res.second);
+      }
+    }
   }
-  for (const auto& v : values.integerStartValues)
+  // set start values from root resources
+  else if (getParentSystem() && getParentSystem()->getValues().hasResources())
   {
-    oms::ComRef cref = getValidCref(v.first);
-    if (oms_status_ok != setInteger(cref, v.second))
-      return logError("Failed to set start value for " + std::string(v.first));
+    setResourcesHelper2(getParentSystem()->getValues());
   }
-  for (const auto& v : values.realStartValues)
+  // set start values from top level root resources
+  else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
   {
-    oms::ComRef cref = getValidCref(v.first);
-    if (oms_status_ok != setReal(cref, v.second))
-      return logError("Failed to set start value for " + std::string(v.first));
+    setResourcesHelper2(getParentSystem()->getParentSystem()->getValues());
+  }
+  // set start values from inline resources
+  else
+  {
+    setResourcesHelper1(values);
   }
 
   // enterInitialization
-  time = getModel()->getStartTime();
+  time = getModel().getStartTime();
   double relativeTolerance = 0.0;
   dynamic_cast<SystemWC*>(getParentSystem())->getTolerance(NULL, &relativeTolerance);
   fmistatus = fmi2_import_setup_experiment(fmu, fmi2_true, relativeTolerance, time, fmi2_false, 1.0);
@@ -518,6 +573,71 @@ oms_status_enu_t oms::ComponentFMUCS::instantiate()
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::setResourcesHelper1(Values values)
+{
+  for (const auto &v : values.booleanStartValues)
+  {
+    oms::ComRef cref = getValidCref(v.first);
+    if (oms_status_ok != setBoolean(cref, v.second))
+      return logError("Failed to set start value for " + std::string(v.first));
+  }
+  for (const auto &v : values.integerStartValues)
+  {
+    oms::ComRef cref = getValidCref(v.first);
+    if (oms_status_ok != setInteger(cref, v.second))
+      return logError("Failed to set start value for " + std::string(v.first));
+  }
+  for (const auto &v : values.realStartValues)
+  {
+    oms::ComRef cref = getValidCref(v.first);
+    if (oms_status_ok != setReal(cref, v.second))
+      return logError("Failed to set start value for " + std::string(v.first));
+  }
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::setResourcesHelper2(Values values)
+{
+  for (const auto &it : values.parameterResources)
+  {
+    for (const auto &res : it.allresources)
+    {
+      for (const auto &v : res.second.booleanStartValues)
+      {
+        oms::ComRef tail(v.first);
+        oms::ComRef head = tail.pop_front();
+        if (head == getCref())
+        {
+          if (oms_status_ok != setBoolean(tail, v.second))
+            return logError("Failed to set start value for " + std::string(v.first));
+        }
+      }
+      for (const auto &v : res.second.integerStartValues)
+      {
+        oms::ComRef tail(v.first);
+        oms::ComRef head = tail.pop_front();
+        if (head == getCref())
+        {
+          if (oms_status_ok != setInteger(tail, v.second))
+            return logError("Failed to set start value for " + std::string(v.first));
+        }
+      }
+      for (const auto &v : res.second.realStartValues)
+      {
+        oms::ComRef tail(v.first);
+        oms::ComRef head = tail.pop_front();
+        if (head == getCref())
+        {
+          if (oms_status_ok != setReal(tail, v.second))
+            return logError("Failed to set start value for " + std::string(v.first));
+        }
+      }
+    }
+  }
+
+  return oms_status_ok;
+}
 
 /*
  * function which returns validCrefs
@@ -535,6 +655,23 @@ oms::ComRef oms::ComponentFMUCS::getValidCref(ComRef cref)
   return tail;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::addResources(std::string& filename)
+{
+  Values resources;
+  if (!values.hasResources())
+  {
+    resources.allresources[filename] = resources;
+    values.parameterResources.push_back(resources);
+  }
+  else
+  {
+    // generate empty ssv file, if more resources are added to same level
+    values.parameterResources[0].allresources[filename] = resources;
+  }
+
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::initialize()
 {
   clock.reset();
@@ -569,7 +706,7 @@ oms_status_enu_t oms::ComponentFMUCS::reset()
     return logError_ResetFailed(getCref());
 
   // enterInitialization
-  time = getModel()->getStartTime();
+  time = getModel().getStartTime();
   double relativeTolerance = 0.0;
   dynamic_cast<SystemWC*>(getParentSystem())->getTolerance(NULL, &relativeTolerance);
   fmistatus = fmi2_import_setup_experiment(fmu, fmi2_true, relativeTolerance, time, fmi2_false, 1.0);
@@ -584,7 +721,7 @@ oms_status_enu_t oms::ComponentFMUCS::reset()
 oms_status_enu_t oms::ComponentFMUCS::stepUntil(double stopTime)
 {
   CallClock callClock(clock);
-  System *topLevelSystem = getModel()->getTopLevelSystem();
+  System *topLevelSystem = getModel().getTopLevelSystem();
 
   fmi2_status_t fmistatus;
   double hdef = (stopTime-time) / 1.0;
@@ -640,26 +777,63 @@ oms_status_enu_t oms::ComponentFMUCS::getBoolean(const ComRef& cref, bool& value
 {
   CallClock callClock(clock);
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
     // check for start values exist, priority over modeldescription.xml start values
-    auto booleanValue = values.booleanStartValues.find(cref);
-    if (booleanValue != values.booleanStartValues.end())
+    if (values.hasResources())  // search in local resources
     {
-      value = booleanValue->second;
-      return oms_status_ok;
-    }
-    else
-    {
-      // search in modelDescription.xml
-      auto booleanValue = values.modelDescriptionBooleanStartValues.find(cref);
-      if (booleanValue != values.modelDescriptionBooleanStartValues.end())
+      if (oms_status_ok == values.getBooleanResources(cref, value, false, oms_modelState_virgin))
       {
-        value = booleanValue->second;
         return oms_status_ok;
       }
+      // search in modelDescription.xml
+      else if (oms_status_ok == values.getBooleanFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())  // search in root resources
+    {
+      if (oms_status_ok == getParentSystem()->getValues().getBooleanResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getBooleanFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())  // search in top level root resources
+    {
+      if (oms_status_ok == getParentSystem()->getParentSystem()->getValues().getBooleanResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getBooleanFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else // search inline
+    {
+      // check for start values exist, priority over modeldescription.xml start values
+      if (oms_status_ok == values.getBoolean(cref, value))
+      {
+        return oms_status_ok;
+      }
+      else
+      {
+        return values.getBooleanFromModeldescription(cref, value);
+      }
     }
-    return logError("no start value provided or available for signal: " + std::string(getFullCref() + cref));
   }
 
   int j=-1;
@@ -693,26 +867,63 @@ oms_status_enu_t oms::ComponentFMUCS::getInteger(const ComRef& cref, int& value)
 {
   CallClock callClock(clock);
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
     // check for start values exist, priority over modeldescription.xml start values
-    auto integerValue = values.integerStartValues.find(cref);
-    if (integerValue != values.integerStartValues.end())
+    if (values.hasResources())  // search in local resources
     {
-      value = integerValue->second;
-      return oms_status_ok;
-    }
-    else
-    {
-      // search in modelDescription.xml
-      auto integerValue = values.modelDescriptionIntegerStartValues.find(cref);
-      if (integerValue != values.modelDescriptionIntegerStartValues.end())
+      if (oms_status_ok == values.getIntegerResources(cref, value, false, oms_modelState_virgin))
       {
-        value = integerValue->second;
         return oms_status_ok;
       }
+      // search in modelDescription.xml
+      else if (oms_status_ok == values.getIntegerFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())  // search in root resources
+    {
+      if (oms_status_ok == getParentSystem()->getValues().getIntegerResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getIntegerFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())  // search in top level root resources
+    {
+      if (oms_status_ok == getParentSystem()->getParentSystem()->getValues().getIntegerResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getIntegerFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else // search inline
+    {
+      // check for start values exist, priority over modeldescription.xml start values
+      if (oms_status_ok == values.getInteger(cref, value))
+      {
+        return oms_status_ok;
+      }
+      else
+      {
+        return values.getIntegerFromModeldescription(cref, value);
+      }
     }
-    return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
   }
 
   int j=-1;
@@ -785,26 +996,63 @@ oms_status_enu_t oms::ComponentFMUCS::getReal(const ComRef& cref, double& value)
 {
   CallClock callClock(clock);
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
     // check for start values exist, priority over modeldescription.xml start values
-    auto realValue = values.realStartValues.find(cref);
-    if (realValue != values.realStartValues.end())
+    if (values.hasResources())  // search in local resources
     {
-      value = realValue->second;
-      return oms_status_ok;
-    }
-    else
-    {
-      // search in modelDescription.xml
-      auto realValue = values.modelDescriptionRealStartValues.find(cref);
-      if (realValue != values.modelDescriptionRealStartValues.end())
+      if (oms_status_ok == values.getRealResources(cref, value, false, oms_modelState_virgin))
       {
-        value = realValue->second;
         return oms_status_ok;
       }
+      // search in modelDescription.xml
+      else if (oms_status_ok == values.getRealFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())  // search in root resources
+    {
+      if (oms_status_ok == getParentSystem()->getValues().getRealResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getRealFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())  // search in top level root resources
+    {
+      if (oms_status_ok == getParentSystem()->getParentSystem()->getValues().getRealResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getRealFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else // search inline
+    {
+      // check for start values exist, priority over modeldescription.xml start values
+      if (oms_status_ok == values.getReal(cref, value))
+      {
+        return oms_status_ok;
+      }
+      else
+      {
+        return values.getRealFromModeldescription(cref, value);
+      }
     }
-    return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
   }
 
   int j=-1;
@@ -890,16 +1138,27 @@ oms_status_enu_t oms::ComponentFMUCS::setBoolean(const ComRef& cref, bool value)
   if (!fmu || j < 0)
     return logError_UnknownSignal(getFullCref() + cref);
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
-    if (Flags::ExportParametersInline())
+    // check for local resources available
+    if (values.hasResources())
     {
-      values.setBoolean(allVariables[j].getCref(), value);
+      return values.setBooleanResources(cref, value, getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in root
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getValues().setBooleanResources(getCref()+cref, value, getParentSystem()->getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in top level root
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getParentSystem()->getValues().setBooleanResources(getCref()+cref, value, getParentSystem()->getParentSystem()->getFullCref(), false, oms_modelState_virgin);
     }
     else
     {
-      // append startValues with prefix (e.g) addP.K1
-      values.setBoolean(getCref()+allVariables[j].getCref(), value);
+      //inline parameter settings
+      values.setBoolean(cref, value);
     }
   }
   else
@@ -929,16 +1188,27 @@ oms_status_enu_t oms::ComponentFMUCS::setInteger(const ComRef& cref, int value)
   if (!fmu || j < 0)
     return logError_UnknownSignal(getFullCref() + cref);
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
-    if (Flags::ExportParametersInline())
+    // check for local resources available
+    if (values.hasResources())
     {
-      values.setInteger(allVariables[j].getCref(), value);
+      return values.setIntegerResources(cref, value, getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in root
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getValues().setIntegerResources(getCref()+cref, value, getParentSystem()->getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in top level root
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getParentSystem()->getValues().setIntegerResources(getCref()+cref, value, getParentSystem()->getParentSystem()->getFullCref(), false, oms_modelState_virgin);
     }
     else
     {
-      // append startValues with prefix (e.g) addP.K1
-      values.setInteger(getCref()+allVariables[j].getCref(), value);
+      //inline parameter settings
+      values.setInteger(cref, value);
     }
   }
   else
@@ -967,20 +1237,31 @@ oms_status_enu_t oms::ComponentFMUCS::setReal(const ComRef& cref, double value)
   if (!fmu || j < 0)
     return logError_UnknownSignal(getFullCref() + cref);
 
-  if (getModel()->validState(oms_modelState_virgin|oms_modelState_enterInstantiation|oms_modelState_instantiated))
+  if (getModel().validState(oms_modelState_virgin|oms_modelState_enterInstantiation|oms_modelState_instantiated))
     if (allVariables[j].isCalculated() || allVariables[j].isIndependent())
       return logWarning("It is not allowed to provide a start value if initial=\"calculated\" or causality=\"independent\".");
 
-  if (oms_modelState_virgin == getModel()->getModelState())
+  if (oms_modelState_virgin == getModel().getModelState())
   {
-    if (Flags::ExportParametersInline())
+    // check for local resources available
+    if (values.hasResources())
     {
-      values.setReal(allVariables[j].getCref(), value);
+      return values.setRealResources(cref, value, getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in root
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getValues().setRealResources(getCref()+cref, value, getParentSystem()->getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in top level root
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getParentSystem()->getValues().setRealResources(getCref()+cref, value, getParentSystem()->getParentSystem()->getFullCref(), false, oms_modelState_virgin);
     }
     else
     {
-      // append startValues with prefix (e.g) addP.K1
-      values.setReal(getCref()+allVariables[j].getCref(), value);
+      //inline parameter settings
+      values.setReal(cref, value);
     }
   }
   else
@@ -995,7 +1276,27 @@ oms_status_enu_t oms::ComponentFMUCS::setReal(const ComRef& cref, double value)
 
 oms_status_enu_t oms::ComponentFMUCS::deleteStartValue(const ComRef& cref)
 {
-  return values.deleteStartValue(cref);
+  // check for local resources
+  if (values.hasResources())
+  {
+    return values.deleteStartValueInResources(cref);
+  }
+  // check for resources in root
+  else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getValues().deleteStartValueInResources(getCref()+cref);
+  }
+  // check for resources in top level root
+  else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getParentSystem()->getValues().deleteStartValueInResources(getCref()+cref);
+  }
+  else
+  {
+    return values.deleteStartValue(cref);
+  }
+
+  return oms_status_error;
 }
 
 oms_status_enu_t oms::ComponentFMUCS::registerSignalsForResultFile(ResultWriter& resultFile)
@@ -1191,3 +1492,37 @@ oms_status_enu_t oms::ComponentFMUCS::setFaultInjection(const oms::ComRef& signa
   }
   return oms_status_ok;
 }
+
+void oms::ComponentFMUCS::getFilteredSignals(std::vector<Connector>& filteredSignals) const
+{
+  for (unsigned int i = 0; i < allVariables.size(); ++i)
+  {
+    if (exportVariables[i])
+      filteredSignals.push_back(allVariables[i].makeConnector(this->getFullCref()));
+  }
+}
+
+oms_status_enu_t oms::ComponentFMUCS::renameValues(const ComRef& oldCref, const ComRef& newCref)
+{
+  // check for local resources
+  if (values.hasResources())
+  {
+    return values.renameInResources(oldCref, newCref);
+  }
+  // check for resources in root
+  else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getValues().renameInResources(oldCref, newCref);
+  }
+  // check for resources in top level root
+  else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getParentSystem()->getValues().renameInResources(oldCref, newCref);
+  }
+  else
+  {
+    return values.rename(oldCref, newCref);
+  }
+
+  return oms_status_error;
+}
