@@ -33,9 +33,10 @@
 
 #include "Logging.h"
 #include "Model.h"
+#include "OMSFileSystem.h"
 #include "ssd/Tags.h"
 #include "System.h"
-#include <OMSFileSystem.h>
+
 #include <RegEx.h>
 
 oms::ComponentTable::ComponentTable(const ComRef& cref, System* parentSystem, const std::string& path)
@@ -71,7 +72,7 @@ oms::Component* oms::ComponentTable::NewComponent(const oms::ComRef& cref, oms::
   if (path.length() > 4)
     extension = path.substr(path.length() - 4);
 
-  filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
+  filesystem::path temp_root(parentSystem->getModel().getTempDirectory());
   filesystem::path relPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + extension)) : filesystem::path(path);
   filesystem::path absPath = temp_root / relPath;
 
@@ -95,7 +96,7 @@ oms::Component* oms::ComponentTable::NewComponent(const oms::ComRef& cref, oms::
   for (auto const &signal : component->resultReader->getAllSignals())
   {
     component->exportSeries[ComRef(signal)] = true;
-    component->connectors.back() = new Connector(oms_causality_output, oms_signal_type_real, ComRef(signal), i++/(double)size);
+    component->connectors.back() = new Connector(oms_causality_output, oms_signal_type_real, ComRef(signal), component->getFullCref(), i++/(double)size);
     component->connectors.push_back(NULL);
   }
   component->element.setConnectors(&component->connectors[0]);
@@ -103,7 +104,7 @@ oms::Component* oms::ComponentTable::NewComponent(const oms::ComRef& cref, oms::
   return component;
 }
 
-oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion)
+oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, oms::System* parentSystem, const std::string& sspVersion, const Snapshot& snapshot)
 {
   ComRef cref = ComRef(node.attribute("name").as_string());
   std::string type = node.attribute("type").as_string();
@@ -136,7 +137,7 @@ oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, om
       // import connectors
       for(pugi::xml_node_iterator itConnectors = (*it).begin(); itConnectors != (*it).end(); ++itConnectors)
       {
-        component->connectors.push_back(oms::Connector::NewConnector(*itConnectors, sspVersion));
+        component->connectors.push_back(oms::Connector::NewConnector(*itConnectors, sspVersion, component->getFullCref()));
         component->exportSeries[component->connectors.back()->getName()] = true;
       }
     }
@@ -160,7 +161,7 @@ oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, om
   return component;
 }
 
-oms_status_enu_t oms::ComponentTable::exportToSSD(pugi::xml_node& node, pugi::xml_node& ssvNode) const
+oms_status_enu_t oms::ComponentTable::exportToSSD(pugi::xml_node& node, Snapshot& snapshot) const
 {
   node.append_attribute("name") = this->getCref().c_str();
   node.append_attribute("type") = "application/table";
@@ -179,13 +180,13 @@ oms_status_enu_t oms::ComponentTable::exportToSSD(pugi::xml_node& node, pugi::xm
 
 oms_status_enu_t oms::ComponentTable::instantiate()
 {
-  time = getModel()->getStartTime();
+  time = getModel().getStartTime();
   return oms_status_ok;
 }
 
 oms_status_enu_t oms::ComponentTable::reset()
 {
-  time = getModel()->getStartTime();
+  time = getModel().getStartTime();
   return oms_status_ok;
 }
 
@@ -380,3 +381,12 @@ oms_status_enu_t oms::ComponentTable::restoreState()
   time = storedTime;
   return oms_status_ok;
 }
+
+void oms::ComponentTable::getFilteredSignals(std::vector<Connector>& filteredSignals) const
+{
+  for (auto& x: exportSeries)
+  {
+    if (x.second)
+      filteredSignals.push_back(Connector(oms_causality_output, oms_signal_type_real, x.first, this->getFullCref()));
+  }
+}
