@@ -43,6 +43,7 @@
 #include "ResultReader.h"
 #include "Scope.h"
 #include "SignalDerivative.h"
+#include "Snapshot.h"
 #include "System.h"
 #include "SystemWC.h"
 #if !defined(NO_TLM)
@@ -132,10 +133,10 @@ oms_status_enu_t oms_rename(const char* cref_, const char* newCref_)
   oms::ComRef cref(cref_);
   oms::ComRef newCref(newCref_);
 
-  if (cref.isValidIdent())
-    return oms::Scope::GetInstance().renameModel(cref, newCref);
-  else
-    return logError_OnlyForModel;
+  if (!newCref.isValidIdent())
+    return logError("The new name \"" + std::string(newCref) + "\" is invalid. It must be a valid (non-qualified) cref.");
+
+  return oms::Scope::GetInstance().renameModel(cref, newCref);
 }
 
 int oms_compareSimulationResults(const char* filenameA, const char* filenameB, const char* var, double relTol, double absTol)
@@ -218,10 +219,16 @@ oms_status_enu_t oms_exportSnapshot(const char* cref_, char** contents)
 {
   oms::ComRef tail(cref_);
   oms::ComRef front = tail.pop_front();
-  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+
+  oms::ComRef modelCref(front);
+  modelCref.pop_suffix();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(modelCref);
   if (!model)
     return logError_ModelNotInScope(front);
 
+  if (tail.isEmpty() && front.hasSuffix())
+    return model->exportSnapshot(oms::ComRef(":" + front.suffix()), contents);
   return model->exportSnapshot(tail, contents);
 }
 
@@ -236,6 +243,17 @@ oms_status_enu_t oms_exportSSVTemplate(const char* cref_, const char* filename)
   return model->exportSSVTemplate(tail, std::string(filename));
 }
 
+oms_status_enu_t oms_exportSSMTemplate(const char * cref_, const char * filename)
+{
+  oms::ComRef tail(cref_);
+  oms::ComRef front = tail.pop_front();
+  oms::Model * model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->exportSSMTemplate(tail, std::string(filename));
+}
+
 oms_status_enu_t oms_listUnconnectedConnectors(const char* cref_, char** contents)
 {
   oms::ComRef tail(cref_);
@@ -251,31 +269,29 @@ oms_status_enu_t oms_listUnconnectedConnectors(const char* cref_, char** content
   return system->listUnconnectedConnectors(contents);
 }
 
-oms_status_enu_t oms_loadSnapshot(const char* cref_, const char* snapshot)
+oms_status_enu_t oms_loadSnapshot(const char* cref, const char* snapshot, char** newCref)
 {
-  oms::ComRef cref(cref_);
-
-  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
-  if (!model)
-    return logError_ModelNotInScope(cref);
-
-  return model->loadSnapshot(snapshot);
+  return oms::Scope::GetInstance().loadSnapshot(oms::ComRef(cref), snapshot, newCref);
 }
 
-oms_status_enu_t oms_importSnapshot(const char* cref_, const char* snapshot)
+oms_status_enu_t oms_importSnapshot(const char* cref, const char* snapshot, char** newCref)
 {
-  oms::ComRef cref(cref_);
-
-  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
-  if (!model)
-    return logError_ModelNotInScope(cref);
-
-  return model->importSnapshot(snapshot);
+  return oms::Scope::GetInstance().importSnapshot(oms::ComRef(cref), snapshot, newCref);
 }
 
-oms_status_enu_t oms_parseModelName(const char* contents, char** cref)
+oms_status_enu_t oms_addResources(const char* cref_)
 {
-  return logError_NotImplemented;
+  oms::ComRef tail(cref_);
+  oms::ComRef front = tail.pop_front();
+
+  oms::ComRef modelCref(front);
+  modelCref.pop_suffix();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(modelCref);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->addResources(tail);
 }
 
 oms_status_enu_t oms_addSystem(const char* cref_, oms_system_enu_t type)
@@ -964,28 +980,6 @@ oms_status_enu_t oms_initialize(const char* cref_)
   return model->initialize();
 }
 
-oms_status_enu_t oms_simulate_asynchronous(const char* cref_, void (*cb)(const char* cref, double time, oms_status_enu_t status))
-{
-  oms::ComRef cref(cref_);
-
-  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
-  if (!model)
-    return logError_ModelNotInScope(cref);
-
-  return model->simulate_asynchronous(cb);
-}
-
-oms_status_enu_t oms_cancelSimulation_asynchronous(const char* cref_)
-{
-  oms::ComRef cref(cref_);
-
-  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
-  if (!model)
-    return logError_ModelNotInScope(cref);
-
-  return model->cancelSimulation_asynchronous();
-}
-
 oms_status_enu_t oms_simulate(const char* cref_)
 {
   oms::ComRef cref(cref_);
@@ -997,6 +991,17 @@ oms_status_enu_t oms_simulate(const char* cref_)
   return model->simulate();
 }
 
+oms_status_enu_t oms_doStep(const char* cref_)
+{
+  oms::ComRef cref(cref_);
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
+  if (!model)
+    return logError_ModelNotInScope(cref);
+
+  return model->doStep();
+}
+
 oms_status_enu_t oms_stepUntil(const char* cref_, double stopTime)
 {
   oms::ComRef cref(cref_);
@@ -1258,22 +1263,6 @@ oms_status_enu_t oms_getResultFile(const char* cref_, char** filename, int* buff
     return logError_OnlyForModel;
 }
 
-oms_status_enu_t oms_getSignalFilter(const char* cref_, char** regex)
-{
-  oms::ComRef cref(cref_);
-
-  if (cref.isValidIdent())
-  {
-    oms::Model* model = oms::Scope::GetInstance().getModel(cref);
-    if (!model)
-      return logError_ModelNotInScope(cref);
-
-    return model->getSignalFilter(regex);
-  }
-  else
-    return logError_OnlyForModel;
-}
-
 oms_status_enu_t oms_getSolver(const char* cref, oms_solver_enu_t* solver)
 {
   oms::ComRef tail(cref);
@@ -1414,24 +1403,8 @@ oms_status_enu_t oms_setResultFile(const char* cref_, const char* filename, int
     return logError_OnlyForModel;
 }
 
-oms_status_enu_t oms_setSignalFilter(const char* cref, const char* regex)
-{
-  oms_status_enu_t status;
-
-  oms::ComRef tail(cref);
-  oms::ComRef front = tail.pop_front();
-
-  oms::Model* model = oms::Scope::GetInstance().getModel(front);
-  if (!model)
-    return logError_ModelNotInScope(front);
-
-  return model->setSignalFilter(regex);
-}
-
 oms_status_enu_t oms_addSignalsToResults(const char* cref, const char* regex)
 {
-  logWarning("[oms_addSignalsToResults] will not update the signalFilters in ssp, use [oms_setSignalFilter]");
-
   oms::ComRef tail(cref);
   oms::ComRef front = tail.pop_front();
 
@@ -1444,8 +1417,6 @@ oms_status_enu_t oms_addSignalsToResults(const char* cref, const char* regex)
 
 oms_status_enu_t oms_removeSignalsFromResults(const char* cref, const char* regex)
 {
-  logWarning("[oms_removeSignalsFromResults] will not update the signalFilters in ssp, use [oms_setSignalFilter]");
-
   oms::ComRef tail(cref);
   oms::ComRef front = tail.pop_front();
 
@@ -1506,6 +1477,19 @@ oms_status_enu_t oms_setStopTime(const char* cref, double stopTime)
   return model->setStopTime(stopTime);
 }
 
+oms_status_enu_t oms_getTime(const char* cref, double* time)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  *time = model->getTime();
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms_setFixedStepSize(const char* cref, double stepSize)
 {
   oms::ComRef tail(cref);
@@ -1643,12 +1627,10 @@ oms_status_enu_t oms_extractFMIKind(const char* filename, oms_fmi_kind_enu_t* ki
   if (status != 0)
     return logError("failed to extract modelDescription.xml from \"" + std::string(filename) + "\"");
 
-  std::string xml_file = oms::Scope::GetInstance().getTempDirectory() + "/modelDescription.xml";
-  pugi::xml_document doc;
-  pugi::xml_parse_result result = doc.load_file(xml_file.c_str());
-  if (!result)
-    return logError("loading \"" + xml_file + "\" failed (" + std::string(result.description()) + ")");
-  const pugi::xml_node node = doc.document_element(); // ssd:SystemStructureDescription
+  oms::Snapshot snapshot;
+  if (oms_status_ok != snapshot.importResourceFile("modelDescription.xml", oms::Scope::GetInstance().getTempDirectory()))
+    return logError("Failed to import");
+  const pugi::xml_node node = snapshot.getResourceNode("modelDescription.xml");
 
   bool cs = (std::string(node.child("CoSimulation").attribute("modelIdentifier").as_string()) != "");
   bool me = (std::string(node.child("ModelExchange").attribute("modelIdentifier").as_string()) != "");
