@@ -294,6 +294,22 @@ static int OMSimulatorLua_oms_exportSSVTemplate(lua_State *L)
   return 1;
 }
 
+//oms_status_enu_t oms_exportSSMTemplate(const char* cref, const char* filename);
+static int OMSimulatorLua_oms_exportSSMTemplate(lua_State *L)
+{
+  if (lua_gettop(L) != 2)
+    return luaL_error(L, "expecting exactly 2 arguments");
+  luaL_checktype(L, 1, LUA_TSTRING);
+  luaL_checktype(L, 2, LUA_TSTRING);
+
+  const char* cref = lua_tostring(L, 1);
+  const char* filename = lua_tostring(L, 2);
+  oms_status_enu_t status = oms_exportSSMTemplate(cref, filename);
+
+  lua_pushinteger(L, status);
+  return 1;
+}
+
 //oms_status_enu_t oms_listUnconnectedConnectors(const char* cref, char** contents);
 static int OMSimulatorLua_oms_listUnconnectedConnectors(lua_State *L)
 {
@@ -314,36 +330,44 @@ static int OMSimulatorLua_oms_listUnconnectedConnectors(lua_State *L)
   return 2;
 }
 
-//oms_status_enu_t oms_loadSnapshot(const char* cref, const char* snapshot);
+//oms_status_enu_t oms_importSnapshot(const char* cref, const char* snapshot, char** newCref);
 static int OMSimulatorLua_oms_loadSnapshot(lua_State *L)
 {
   if (lua_gettop(L) != 2)
-    return luaL_error(L, "expecting exactly 2 arguments\n<integer> = oms_loadSnapshot(<string>, <string>)");
+    return luaL_error(L, "expecting exactly 2 arguments\n<string>, <integer> = oms_loadSnapshot(<string>, <string>)");
   luaL_checktype(L, 1, LUA_TSTRING);
   luaL_checktype(L, 2, LUA_TSTRING);
 
   const char* cref = lua_tostring(L, 1);
   const char* snapshot = lua_tostring(L, 2);
-  oms_status_enu_t status = oms_loadSnapshot(cref, snapshot);
 
+  char* newCref = NULL;
+  oms_status_enu_t status = oms_loadSnapshot(cref, snapshot, &newCref);
+
+  lua_pushstring(L, newCref ? newCref : "");
   lua_pushinteger(L, status);
-  return 1;
+
+  return 2;
 }
 
-//oms_status_enu_t oms_importSnapshot(const char* cref, const char* snapshot);
+//oms_status_enu_t oms_importSnapshot(const char* cref, const char* snapshot, char** newCref);
 static int OMSimulatorLua_oms_importSnapshot(lua_State *L)
 {
   if (lua_gettop(L) != 2)
-    return luaL_error(L, "expecting exactly 2 arguments\n<integer> = oms_loadSnapshot(<string>, <string>)");
+    return luaL_error(L, "expecting exactly 2 arguments\n<integer> = oms_importSnapshot(<string>, <string>)");
   luaL_checktype(L, 1, LUA_TSTRING);
   luaL_checktype(L, 2, LUA_TSTRING);
 
   const char* cref = lua_tostring(L, 1);
   const char* snapshot = lua_tostring(L, 2);
-  oms_status_enu_t status = oms_importSnapshot(cref, snapshot);
 
+  char* newCref = NULL;
+  oms_status_enu_t status = oms_importSnapshot(cref, snapshot, &newCref);
+
+  lua_pushstring(L, newCref ? newCref : "");
   lua_pushinteger(L, status);
-  return 1;
+
+  return 2;
 }
 
 //oms_status_enu_t oms_exportDependencyGraphs(const char* cref, const char* initialization, const char* event, const char* simulation);
@@ -367,26 +391,6 @@ static int OMSimulatorLua_oms_exportDependencyGraphs(lua_State *L)
   return 1;
 }
 
-//oms_status_enu_t oms_parseModelName(const char* contents, char** cref);
-static int OMSimulatorLua_oms_parseModelName(lua_State *L)
-{
-  if (lua_gettop(L) != 1)
-    return luaL_error(L, "expecting exactly 1 argument");
-  luaL_checktype(L, 1, LUA_TSTRING);
-
-  const char* contents = lua_tostring(L, 1);
-  char* cref = NULL;
-  oms_status_enu_t status = oms_parseModelName(contents, &cref);
-
-  lua_pushstring(L, cref ? cref : "");
-  lua_pushinteger(L, status);
-
-  if (cref)
-    oms_freeMemory(cref);
-
-  return 2;
-}
-
 //int oms_compareSimulationResults(const char* filenameA, const char* filenameB, const char* var, double relTol, double absTol);
 static int OMSimulatorLua_oms_compareSimulationResults(lua_State *L)
 {
@@ -408,6 +412,20 @@ static int OMSimulatorLua_oms_compareSimulationResults(lua_State *L)
   return 1;
 }
 
+//oms_status_enu_t oms_addResources(const char* cref);
+static int OMSimulatorLua_oms_addResources(lua_State *L)
+{
+  if (lua_gettop(L) != 1)
+    return luaL_error(L, "expecting exactly 1 argument");
+  luaL_checktype(L, 1, LUA_TSTRING);
+
+  const char* cref = lua_tostring(L, 1);
+  oms_status_enu_t status = oms_addResources(cref);
+
+  lua_pushinteger(L, status);
+  return 1;
+}
+
 //oms_status_enu_t oms_addSystem(const char* cref, oms_system_enu_t type);
 static int OMSimulatorLua_oms_addSystem(lua_State *L)
 {
@@ -1126,21 +1144,6 @@ static int OMSimulatorLua_oms_setResultFile(lua_State *L)
   return 1;
 }
 
-//oms_status_enu_t oms_setSignalFilter(const char* cref, const char* regex)
-static int OMSimulatorLua_oms_setSignalFilter(lua_State *L)
-{
-  if (lua_gettop(L) != 2)
-    return luaL_error(L, "expecting exactly 2 argument");
-  luaL_checktype(L, 1, LUA_TSTRING);
-  luaL_checktype(L, 2, LUA_TSTRING);
-
-  const char* cref = lua_tostring(L, 1);
-  const char* regex = lua_tostring(L, 2);
-  oms_status_enu_t status = oms_setSignalFilter(cref, regex);
-  lua_pushinteger(L, status);
-  return 1;
-}
-
 //OMSAPI oms_status_enu_t OMSCALL oms_setSolver(const char* cref, oms_solver_enu_t solver);
 static int OMSimulatorLua_oms_setSolver(lua_State *L)
 {
@@ -1269,6 +1272,7 @@ DLLEXPORT int luaopen_OMSimulatorLua(lua_State *L)
   REGISTER_LUA_CALL(oms_addDynamicValueIndicator);
   REGISTER_LUA_CALL(oms_addEventIndicator);
   REGISTER_LUA_CALL(oms_addExternalModel);
+  REGISTER_LUA_CALL(oms_addResources);
   REGISTER_LUA_CALL(oms_addSignalsToResults);
   REGISTER_LUA_CALL(oms_addStaticValueIndicator);
   REGISTER_LUA_CALL(oms_addSubModel);
@@ -1286,6 +1290,7 @@ DLLEXPORT int luaopen_OMSimulatorLua(lua_State *L)
   REGISTER_LUA_CALL(oms_exportDependencyGraphs);
   REGISTER_LUA_CALL(oms_exportSnapshot);
   REGISTER_LUA_CALL(oms_exportSSVTemplate);
+  REGISTER_LUA_CALL(oms_exportSSMTemplate);
   REGISTER_LUA_CALL(oms_faultInjection);
   REGISTER_LUA_CALL(oms_getBoolean);
   REGISTER_LUA_CALL(oms_getFixedStepSize);
@@ -1306,7 +1311,6 @@ DLLEXPORT int luaopen_OMSimulatorLua(lua_State *L)
   REGISTER_LUA_CALL(oms_listUnconnectedConnectors);
   REGISTER_LUA_CALL(oms_loadSnapshot);
   REGISTER_LUA_CALL(oms_newModel);
-  REGISTER_LUA_CALL(oms_parseModelName);
   REGISTER_LUA_CALL(oms_removeSignalsFromResults);
   REGISTER_LUA_CALL(oms_rename);
   REGISTER_LUA_CALL(oms_reset);
@@ -1321,7 +1325,6 @@ DLLEXPORT int luaopen_OMSimulatorLua(lua_State *L)
   REGISTER_LUA_CALL(oms_setReal);
   REGISTER_LUA_CALL(oms_setRealInputDerivative);
   REGISTER_LUA_CALL(oms_setResultFile);
-  REGISTER_LUA_CALL(oms_setSignalFilter);
   REGISTER_LUA_CALL(oms_setSolver);
   REGISTER_LUA_CALL(oms_setStartTime);
   REGISTER_LUA_CALL(oms_setStopTime);
